from insights.tests import context_wrap
from insights.parsers import foreman_log
from insights.parsers.foreman_log import SatelliteLog, ProductionLog
from insights.parsers.foreman_log import CandlepinLog, ProxyLog
from insights.parsers.foreman_log import CandlepinErrorLog
from insights.parsers.foreman_log import ForemanSSLAccessLog
from datetime import datetime
import doctest


PRODUCTION_LOG = """
2015-11-13 03:30:07 [I] Completed 200 OK in 1783ms (Views: 0.2ms | ActiveRecord: 172.9ms)
2015-11-13 03:30:07 [I] Processing by Katello::Api::V2::RepositoriesController#sync_complete as JSON
2015-11-13 03:30:07 [I]   Parameters: {"call_report"=>"[FILTERED]", "event_type"=>"repo.sync.finish", "payload"=>{"importer_id"=>"yum_importer", "exception"=>nil, "repo_id"=>"1-Acme_Corp_-Red_Hat_Enterprise_Linux_Server-Red_Hat_Satellite_Tools_6_1_for_RHEL_6_Server_RPMs_i386", "traceback"=>nil, "started"=>"2015-11-13T08:30:00Z", "_ns"=>"repo_sync_results", "completed"=>"2015-11-13T08:30:06Z", "importer_type_id"=>"yum_importer", "error_message"=>nil, "summary"=>{"content"=>{"state"=>"FINISHED"}, "comps"=>{"state"=>"FINISHED"}, "distribution"=>{"state"=>"FINISHED"}, "errata"=>{"state"=>"FINISHED"}, "metadata"=>{"state"=>"FINISHED"}}, "added_count"=>0, "result"=>"success", "updated_count"=>3, "details"=>{"content"=>{"size_total"=>0, "items_left"=>0, "items_total"=>0, "state"=>"FINISHED", "size_left"=>0, "details"=>{"rpm_total"=>0, "rpm_done"=>0, "drpm_total"=>0, "drpm_done"=>0}, "error_details"=>[]}, "comps"=>{"state"=>"FINISHED"}, "distribution"=>{"items_total"=>0, "state"=>"FINISHED", "error_details"=>[], "items_left"=>0}, "errata"=>{"state"=>"FINISHED"}, "metadata"=>{"state"=>"FINISHED"}}, "id"=>"56459f8ef301a213bbfd87bb", "removed_count"=>0}, "token"=>"oQumn3XsKrdRkijuvpCNhKF2PDWZt6az", "api_version"=>"v2", "repository"=>{}}
2015-11-13 03:30:07 [I] Sync_complete called for Red Hat Satellite Tools 6.1 for RHEL 6 Server RPMs i386, running after_sync.
2015-11-13 03:30:09 [I] Completed 200 OK in 1995ms (Views: 0.2ms | ActiveRecord: 81.5ms)
2015-11-13 03:30:10 [I] Processing by Katello::Api::V2::RepositoriesController#sync_complete as JSON
2015-11-13 03:30:10 [I]   Parameters: {"call_report"=>"[FILTERED]", "event_type"=>"repo.sync.finish", "payload"=>{"importer_id"=>"yum_importer", "exception"=>nil, "repo_id"=>"1-Acme_Corp_-Red_Hat_Enterprise_Linux_Server-Red_Hat_Satellite_Tools_6_1_for_RHEL_5_Server_RPMs_i386", "traceback"=>nil, "started"=>"2015-11-13T08:30:05Z", "_ns"=>"repo_sync_results", "completed"=>"2015-11-13T08:30:10Z", "importer_type_id"=>"yum_importer", "error_message"=>nil, "summary"=>{"content"=>{"state"=>"FINISHED"}, "comps"=>{"state"=>"FINISHED"}, "distribution"=>{"state"=>"FINISHED"}, "errata"=>{"state"=>"FINISHED"}, "metadata"=>{"state"=>"FINISHED"}}, "added_count"=>0, "result"=>"success", "updated_count"=>3, "details"=>{"content"=>{"size_total"=>0, "items_left"=>0, "items_total"=>0, "state"=>"FINISHED", "size_left"=>0, "details"=>{"rpm_total"=>0, "rpm_done"=>0, "drpm_total"=>0, "drpm_done"=>0}, "error_details"=>[]}, "comps"=>{"state"=>"FINISHED"}, "distribution"=>{"items_total"=>0, "state"=>"FINISHED", "error_details"=>[], "items_left"=>0}, "errata"=>{"state"=>"FINISHED"}, "metadata"=>{"state"=>"FINISHED"}}, "id"=>"56459f92f301a2137cd6b802", "removed_count"=>0}, "token"=>"oQumn3XsKrdRkijuvpCNhKF2PDWZt6az", "api_version"=>"v2", "repository"=>{}}
2015-11-13 03:30:10 [I] Sync_complete called for Red Hat Satellite Tools 6.1 for RHEL 5 Server RPMs i386, running after_sync.
2015-11-13 03:30:11 [I] Connecting to database specified by database.yml
2015-11-13 03:30:11 [I] Connecting to database specified by database.yml
2015-11-13 03:30:11 [I] Completed 200 OK in 818ms (Views: 0.2ms | ActiveRecord: 77.2ms)
2015-11-13 03:30:17 [I] Connecting to database specified by database.yml
2015-11-13 03:30:26 [I] Sync_complete called for RHN Tools for Red Hat Enterprise Linux 5 Server RPMs x86_64 5Server, running after_sync.
2015-11-13 03:50:46 [I] Completed 200 OK in 2583ms (Views: 2.7ms | ActiveRecord: 0.3ms)
2015-11-13 06:58:25 [I]   Parameters: {"id"=>"cfd7275b-8cce-4323-8d1f-55ef85eca883"}
2015-11-13 06:58:25 [I] Completed 200 OK in 249ms (Views: 3.1ms | ActiveRecord: 0.3ms)
2015-11-13 06:59:26 [I] Processing by Katello::Api::Rhsm::CandlepinProxiesController#consumer_show as JSON
2015-11-13 06:59:26 [I]   Parameters: {"id"=>"cfd7275b-8cce-4323-8d1f-55ef85eca883"}
2015-11-13 06:59:26 [I] Completed 200 OK in 84ms (Views: 3.1ms | ActiveRecord: 0.3ms)
2015-11-13 07:00:12 [I] Connecting to database specified by database.yml
2015-11-13 07:00:12 [I] Connecting to database specified by database.yml
2015-11-13 07:00:12 [I] Connecting to database specified by database.yml
2015-11-13 07:00:18 [W] Creating scope :completer_scope. Overwriting existing method Organization.completer_scope.
2015-11-13 07:00:18 [W] Creating scope :completer_scope. Overwriting existing method Organization.completer_scope.
2015-11-13 07:00:18 [W] Creating scope :completer_scope. Overwriting existing method Location.completer_scope.
2015-11-13 07:00:18 [W] Creating scope :completer_scope. Overwriting existing method Location.completer_scope.
2015-11-13 07:00:18 [W] Creating scope :completer_scope. Overwriting existing method Organization.completer_scope.
2015-11-13 07:09:22 [I]   Parameters: {"facts"=>"[FILTERED]", "name"=>"infrhnpl002.example.com", "certname"=>"infrhnpl002.example.com", "apiv"=>"v2", :host=>{"name"=>"infrhnpl002.example.com", "certname"=>"infrhnpl002.example.com"}}
2015-11-13 07:09:22 [I] Import facts for 'infrhnpl002.example.com' completed. Added: 0, Updated: 6, Deleted 0 facts
2015-11-13 07:09:22 [I] Completed 201 Created in 251ms (Views: 179.3ms | ActiveRecord: 0.0ms)
2015-11-13 07:09:22 [I] Processing by HostsController#externalNodes as YML
2015-11-13 07:09:22 [I]   Parameters: {"name"=>"infrhnpl002.example.com"}
2015-11-13 07:09:22 [I]   Rendered text template (0.0ms)
2015-11-13 07:09:22 [I] Completed 200 OK in 48ms (Views: 0.5ms | ActiveRecord: 6.6ms)
2015-11-13 07:09:22 [I] Processing by Api::V2::ReportsController#create as JSON
2015-11-13 07:09:22 [I]   Parameters: {"report"=>"[FILTERED]", "apiv"=>"v2"}
2015-11-13 07:09:22 [I]   Rendered text template (0.0ms)
2015-11-13 07:09:22 [I] processing report for infrhnpl002.example.com
2015-11-13 07:09:22 [I] Imported report for infrhnpl002.example.com in 0.02 seconds
2015-11-13 07:09:22 [I] Completed 201 Created in 28ms (Views: 1.2ms | ActiveRecord: 0.0ms)
2015-11-13 07:30:17 [W] Creating scope :completer_scope. Overwriting existing method Organization.completer_scope.
2015-11-13 07:30:18 [W] Creating scope :completer_scope. Overwriting existing method Location.completer_scope.
2015-11-13 07:30:18 [W] Creating scope :completer_scope. Overwriting existing method Location.completer_scope.
2015-11-13 07:30:18 [W] Creating scope :completer_scope. Overwriting existing method Location.completer_scope.
2015-11-13 07:30:25 [I] Client connected.
2015-11-13 07:30:25 [I] Connected to server.
2015-11-13 07:30:25 [I] Client connected.
2015-11-13 07:30:25 [I] Connected to server.
2015-11-13 07:30:25 [I] Client connected.
2015-11-13 07:30:25 [I] Connected to server.
2015-11-13 07:30:30 [I] init config for SecureHeaders::Configuration
2015-11-13 07:30:30 [I] init config for SecureHeaders::Configuration
2015-11-13 07:30:30 [I] init config for SecureHeaders::Configuration
2015-11-13 07:30:32 [I] Processing by Katello::Api::Rhsm::CandlepinProxiesController#consumer_show as JSON
2015-11-13 07:30:32 [I]   Parameters: {"id"=>"cfd7275b-8cce-4323-8d1f-55ef85eca883"}
2015-11-13 07:30:32 [I] Completed 200 OK in 110ms (Views: 2.7ms | ActiveRecord: 0.3ms)
2015-11-13 07:30:33 [I] 2015-11-13 07:30:33 -0500: Expired 48 Reports
2015-11-13 07:30:33 [I] Client disconnected.
2015-11-13 09:41:58 [I] Completed 200 OK in 93ms (Views: 2.9ms | ActiveRecord: 0.3ms)
2015-11-13 09:42:58 [I] Processing by Katello::Api::Rhsm::CandlepinProxiesController#consumer_show as JSON
2015-11-13 09:42:58 [I]   Parameters: {"id"=>"cfd7275b-8cce-4323-8d1f-55ef85eca883"}
2015-11-13 09:42:58 [I] Completed 200 OK in 80ms (Views: 3.6ms | ActiveRecord: 0.3ms)
2015-11-13 09:43:58 [I] Processing by Katello::Api::Rhsm::CandlepinProxiesController#consumer_show as JSON
2015-11-13 09:43:58 [I]   Parameters: {"id"=>"cfd7275b-8cce-4323-8d1f-55ef85eca883"}
2015-11-13 09:43:59 [I] Completed 200 OK in 80ms (Views: 2.9ms | ActiveRecord: 0.3ms)
""".strip()


SATELLITE_OUT = """
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config::Pulp_client/Foreman_config_entry[pulp_client_cert]/require: requires Class[Certs::Pulp_client]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config::Pulp_client/Foreman_config_entry[pulp_client_cert]/require: requires Exec[foreman-rake-db:seed]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config::Pulp_client/Foreman_config_entry[pulp_client_key]/require: requires Class[Certs::Pulp_client]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config::Pulp_client/Foreman_config_entry[pulp_client_key]/require: requires Exec[foreman-rake-db:seed]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config/File[/etc/foreman/plugins/katello.yaml]/before: requires Class[Foreman::Database]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config/File[/etc/foreman/plugins/katello.yaml]/before: requires Exec[foreman-rake-db:migrate]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config/File[/etc/foreman/plugins/katello.yaml]/notify: subscribes to Service[foreman-tasks]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config/File[/etc/foreman/plugins/katello.yaml]/notify: subscribes to Class[Foreman::Service]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Katello::Config/Foreman::Config::Passenger::Fragment[katello]/require: requires Class[Foreman::Config::Passenger]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/notify: subscribes to Class[Certs::Candlepin]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Cert[kam1opapp999.example2.com-qpid-broker]/notify: subscribes to Pubkey[/etc/pki/katello/certs/kam1opapp999.example2.com-qpid-broker.crt]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Pubkey[/etc/pki/katello/certs/kam1opapp999.example2.com-qpid-broker.crt]/notify: subscribes to Privkey[/etc/pki/katello/private/kam1opapp999.example2.com-qpid-broker.key]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Privkey[/etc/pki/katello/private/kam1opapp999.example2.com-qpid-broker.key]/notify: subscribes to File[/etc/pki/katello/private/kam1opapp999.example2.com-qpid-broker.key]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/File[/etc/pki/katello/private/kam1opapp999.example2.com-qpid-broker.key]/notify: subscribes to File[/etc/pki/katello/nssdb]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/File[/etc/pki/katello/nssdb]/notify: subscribes to Exec[generate-nss-password]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Exec[generate-nss-password]/before: requires File[/etc/pki/katello/nssdb/nss_db_password-file]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/File[/etc/pki/katello/nssdb/nss_db_password-file]/notify: subscribes to Exec[create-nss-db]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Exec[create-nss-db]/before: requires Exec[delete ca]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Exec[create-nss-db]/before: requires Exec[delete broker]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Exec[create-nss-db]/before: requires Exec[delete amqp-client]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Exec[create-nss-db]/notify: subscribes to Certs::Ssltools::Certutil[ca]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Certs::Ssltools::Certutil[ca]/notify: subscribes to File[/etc/pki/katello/nssdb/cert8.db]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Certs::Ssltools::Certutil[ca]/notify: subscribes to File[/etc/pki/katello/nssdb/key3.db]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Certs::Ssltools::Certutil[ca]/notify: subscribes to File[/etc/pki/katello/nssdb/secmod.db]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/File[/etc/pki/katello/nssdb/cert8.db]/notify: subscribes to Certs::Ssltools::Certutil[broker]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/File[/etc/pki/katello/nssdb/key3.db]/notify: subscribes to Certs::Ssltools::Certutil[broker]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/File[/etc/pki/katello/nssdb/secmod.db]/notify: subscribes to Certs::Ssltools::Certutil[broker]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Certs::Ssltools::Certutil[broker]/notify: subscribes to Exec[generate-pfx-for-nss-db]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Exec[generate-pfx-for-nss-db]/notify: subscribes to Exec[add-private-key-to-nss-db]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Qpid/Exec[add-private-key-to-nss-db]/notify: subscribes to Service[qpidd]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/notify: subscribes to Class[Candlepin]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Cert[java-client]/notify: subscribes to Pubkey[/etc/pki/katello/certs/java-client.crt]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/File[/etc/pki/katello/keystore_password-file]/notify: subscribes to Exec[candlepin-generate-ssl-keystore]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Exec[candlepin-generate-ssl-keystore]/notify: subscribes to File[/usr/share/tomcat/conf/keystore]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/File[/usr/share/tomcat/conf/keystore]/notify: subscribes to Service[tomcat]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Pubkey[/etc/pki/katello/certs/java-client.crt]/notify: subscribes to Privkey[/etc/pki/katello/private/java-client.key]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Privkey[/etc/pki/katello/private/java-client.key]/notify: subscribes to Certs::Ssltools::Certutil[amqp-client]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Certs::Ssltools::Certutil[amqp-client]/subscribe: subscribes to Exec[create-nss-db]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Certs::Ssltools::Certutil[amqp-client]/notify: subscribes to Service[qpidd]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Certs::Ssltools::Certutil[amqp-client]/notify: subscribes to File[/etc/candlepin/certs/amqp]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/File[/etc/candlepin/certs/amqp]/notify: subscribes to Exec[create candlepin qpid exchange]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Exec[create candlepin qpid exchange]/require: requires Service[qpidd]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Exec[create candlepin qpid exchange]/notify: subscribes to Exec[import CA into Candlepin truststore]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Exec[import CA into Candlepin truststore]/notify: subscribes to Exec[import client certificate into Candlepin keystore]
[DEBUG 2016-08-11 13:09:49 main]  /Stage[main]/Certs::Candlepin/Exec[import client certificate into Candlepin keystore]/notify: subscribes to File[/etc/candlepin/certs/amqp/candlepin.jks]
[DEBUG 2016-08-11 13:09:50 main]  /Stage[main]/Certs::Candlepin/File[/etc/candlepin/certs/amqp/candlepin.jks]/notify: subscribes to Service[tomcat]
[DEBUG 2016-08-11 13:09:50 main]  /Stage[main]/Candlepin/notify: subscribes to Class[Qpid]
[DEBUG 2016-08-11 13:09:51 main]  /Stage[main]/Candlepin::Install/notify: subscribes to Class[Candlepin::Config]
[DEBUG 2016-08-11 13:09:51 main]  /Stage[main]/Candlepin::Config/notify: subscribes to Class[Candlepin::Database]
[DEBUG 2016-08-11 13:09:52 main]  /Stage[main]/Candlepin::Database/notify: subscribes to Class[Candlepin::Service]
""".strip()

CANDLEPIN_LOG = """
2016-09-09 13:45:52,650 [req=bd5a4284-d280-4fc5-a3d5-fc976b7aa5cc, org=] INFO org.candlepin.common.filter.LoggingFilter - Request: verb=GET, uri=/candlepin/consumers/f7677b4b-c470-4626-86a4-2fdf2546af4b
2016-09-09 13:45:52,784 [req=bd5a4284-d280-4fc5-a3d5-fc976b7aa5cc, org=example_org] INFO  org.candlepin.common.filter.LoggingFilter - Response: status=200, content-type="application/json", time=134
2016-09-09 13:45:52,947 [req=909ca4c5-f24e-4212-8f23-cc754d06ac57, org=] INFO org.candlepin.common.filter.LoggingFilter - Request: verb=GET, uri=/candlepin/consumers/f7677b4b-c470-4626-86a4-2fdf2546af4b/content_overrides
2016-09-09 13:45:52,976 [req=909ca4c5-f24e-4212-8f23-cc754d06ac57, org=] INFO org.candlepin.common.filter.LoggingFilter - Response: status=200, content-type="application/json", time=29
2016-09-09 13:45:53,072 [req=49becd26-5dfe-4d2f-8667-470519230d88, org=] INFO org.candlepin.common.filter.LoggingFilter - Request: verb=GET, uri=/candlepin/consumers/f7677b4b-c470-4626-86a4-2fdf2546af4b/release
2016-09-09 13:45:53,115 [req=49becd26-5dfe-4d2f-8667-470519230d88, org=example_org] INFO  org.candlepin.common.filter.LoggingFilter - Response: status=200, content-type="application/json", time=43
""".strip()


PROXY_LOG = """
127.0.0.1 - - [31/May/2016:09:42:28 -0400] "GET /puppet/environments/KT_Encore_Library_RHEL_6_5/classes HTTP/1.1" 200 76785 6.1205
127.0.0.1 - - [31/May/2016:09:42:38 -0400] "GET /puppet/environments/KT_Encore_Library_RHEL_7_6/classes HTTP/1.1" 200 76785 4.4754
127.0.0.1 - - [31/May/2016:09:42:49 -0400] "GET /puppet/environments/KT_Encore_Library_RHEL6_8/classes HTTP/1.1" 200 76785 4.5776
127.0.0.1 - - [31/May/2016:09:57:34 -0400] "GET /tftp/serverName HTTP/1.1" 200 38 0.0014
E, [2016-05-31T09:57:35.884636 #4494] ERROR -- : Record 172.16.100.0/172.16.100.17 not found ]
""".strip()


CANDLEPIN_ERROR_LOG = """
2016-09-07 13:56:49,001 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
2016-09-07 14:07:33,735 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
2016-09-07 14:09:55,173 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
2016-09-07 15:20:33,796 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
2016-09-07 15:27:34,367 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
2016-09-07 16:49:24,650 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
2016-09-07 18:07:53,688 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.sync.Importer - Conflicts occurred during import that were not overridden:
2016-09-07 18:07:53,690 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.sync.Importer - [DISTRIBUTOR_CONFLICT]
2016-09-07 18:07:53,711 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.resource.OwnerResource - Recording import failure org.candlepin.sync.ImportConflictException: Owner has already imported from another subscription management application.
""".strip()


FOREMAN_SSL_ACCESS_SSL_LOG = """
10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/385e688f-43ad-41b2-9fc7-593942ddec78 HTTP/1.1" 200 10736 "-" "-"
10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/status HTTP/1.1" 200 263 "-" "-"
10.185.73.33 - 8a31cd915917666001591d6fb44602a7 [27/Mar/2017:13:34:52 -0400] "GET /pulp/repos/Acme_Inc/Library/RHEL7_Sat_Capsule_Servers/content/dist/rhel/server/7/7Server/x86_64/os/repodata/repomd.xml HTTP/1.1" 200 2018 "-" "urlgrabber/3.10 yum/3.4.3"
10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/4f8a39d0-38b6-4663-8b7e-03368be4d3ab/owner HTTP/1.1" 200 5159 "-" "-"
10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/385e688f-43ad-41b2-9fc7-593942ddec78/compliance HTTP/1.1" 200 5527 "-" "-"
10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/4f8a39d0-38b6-4663-8b7e-03368be4d3ab HTTP/1.1" 200 10695 "-" "-"
10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/385e688f-43ad-41b2-9fc7-593942ddec78/entitlements?exclude=certificates.key&exclude=certificates.cert HTTP/1.1" 200 9920 "-" "-"
""".strip()


def test_production_log():
    fm_log = ProductionLog(context_wrap(PRODUCTION_LOG))
    assert 2 == len(fm_log.get("Rendered text template"))
    line_dict = fm_log.get("Katello::Api::V2::RepositoriesController#sync_complete")[0]
    assert line_dict["message"] == \
        "[I] Processing by Katello::Api::V2::RepositoriesController#sync_complete as JSON"
    assert line_dict["timestamp"] == datetime(2015, 11, 13, 3, 30, 7)
    assert "Expired 48 Reports" in fm_log
    assert fm_log.get("Completed 200 OK in 93")[0]['raw_message'] == \
        "2015-11-13 09:41:58 [I] Completed 200 OK in 93ms (Views: 2.9ms | ActiveRecord: 0.3ms)"
    assert len(list(fm_log.get_after(datetime(2015, 11, 13, 9, 41, 58)))) == 7


def test_proxy_log():
    px_log = ProxyLog(context_wrap(PROXY_LOG))
    assert "ERROR -- " in px_log
    assert len(px_log.get("KT_Encore_Library_RHEL")) == 3
    # Test selection by both time formats - regular line format
    assert len(list(px_log.get_after(datetime(2016, 5, 31, 9, 45, 0)))) == 2
    # ... and error format
    assert len(list(px_log.get_after(datetime(2016, 5, 31, 9, 57, 35)))) == 1


def test_candlepin_log():
    cp_log = CandlepinLog(context_wrap(CANDLEPIN_LOG))
    assert "req=49becd26-5dfe-4d2f-8667-470519230d88" in cp_log
    assert len(cp_log.get("req=bd5a4284-d280-4fc5-a3d5-fc976b7aa5cc")) == 2
    # https://github.com/RedHatInsights/insights-core/pull/2641
    # assert len(list(cp_log.get_after(datetime(2016, 9, 9, 13, 45, 53)))) == 2
    assert cp_log.get("req=bd5a4284-d280-4fc5-a3d5-fc976b7aa5cc")[0]['timestamp'] == datetime(2016, 9, 9, 13, 45, 52, 650000)


def test_satellite_log():
    sat_log = SatelliteLog(context_wrap(SATELLITE_OUT))
    assert "subscribes to Class[Qpid]" in sat_log
    assert len(sat_log.get("notify: subscribes to Class[")) == 7
    assert len(list(sat_log.get_after(datetime(2016, 8, 11, 13, 9, 50)))) == 5


def test_candlepin_error_log():
    error_log = CandlepinErrorLog(context_wrap(CANDLEPIN_ERROR_LOG))
    assert "req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28" in error_log
    assert len(error_log.get("req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28")) == 3
    assert len(list(error_log.get_after(datetime(2016, 9, 7, 18, 7, 53)))) == 3
    assert len(list(error_log.get_after(datetime(2016, 9, 7, 16, 0, 0)))) == 4


def test_foreman_ssl_access_ssl_log():
    foreman_ssl_access_log = ForemanSSLAccessLog(context_wrap(FOREMAN_SSL_ACCESS_SSL_LOG))
    assert "385e688f-43ad-41b2-9fc7-593942ddec78" in foreman_ssl_access_log
    assert len(foreman_ssl_access_log.get("GET /rhsm/consumers")) == 5
    assert len(foreman_ssl_access_log.get("385e688f-43ad-41b2-9fc7-593942ddec78")) == 3
    assert len(list(foreman_ssl_access_log.get_after(datetime(2017, 3, 27, 13, 34, 0)))) == 7


def test_doc():
    failed_count, tests = doctest.testmod(foreman_log,
        globs={"cp_log": CandlepinLog(context_wrap(CANDLEPIN_LOG)),
               "candlepin_log": CandlepinErrorLog(context_wrap(CANDLEPIN_ERROR_LOG)),
               "foreman_ssl_acess_log": ForemanSSLAccessLog(context_wrap(FOREMAN_SSL_ACCESS_SSL_LOG))})
    assert failed_count == 0



"""
LpstatPrinters - command ``lpstat -p``
======================================

Parses the output of ``lpstat -p``, to get locally configured printers.

Current available printer states are:

* IDLE (``PRINTER_STATUS_IDLE``)
* PROCESSING (``PRINTER_STATUS_PROCESSING``) -- printing
* DISABLED (``PRINTER_STATUS_DISABLED``)
* UNKNOWN (``PRINTER_STATUS_UNKNOWN``)


Examples:
    >>> from insights.parsers.lpstat import LpstatPrinters, PRINTER_STATUS_DISABLED
    >>> from insights.tests import context_wrap
    >>> LPSTAT_P_OUTPUT = '''
    ... printer idle_printer is idle.  enabled since Fri 20 Jan 2017 09:55:50 PM CET
    ... printer disabled_printer disabled since Wed 15 Feb 2017 12:01:11 PM EST -
    ...     reason unknown
    ... '''
    >>> lpstat = LpstatPrinters(context_wrap(LPSTAT_P_OUTPUT))
    >>> lpstat.printers
    [{'status': 'IDLE', 'name': 'idle_printer'}, {'status': 'DISABLED', 'name': 'disabled_printer'}]
    >>> lpstat.printer_names_by_status(PRINTER_STATUS_DISABLED)
    ['disabled_printer']

"""

from .. import parser, CommandParser
from insights.specs import Specs

# Printer states
PRINTER_STATUS_IDLE = 'IDLE'
PRINTER_STATUS_PROCESSING = 'PROCESSING'
PRINTER_STATUS_DISABLED = 'DISABLED'
PRINTER_STATUS_UNKNOWN = 'UNKNOWN'

START_LINE_MARKER = 'printer '


@parser(Specs.lpstat_p)
class LpstatPrinters(CommandParser):
    """Class to parse ``lpstat -p`` command output.

    Raises:
        ValueError: Raised if any error occurs parsing the content.
    """

    def __init__(self, *args, **kwargs):
        self.printers = []
        """dict: Dictionary of locally configured printers, with keys 'name' and 'status'"""
        super(LpstatPrinters, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        marker_len = len(START_LINE_MARKER)
        for line in content:
            if line.startswith(START_LINE_MARKER):
                printer_line = line[marker_len:]
                # cut printer name until next space character
                printer_name = printer_line[:printer_line.index(' ')]
                state_line_starts = marker_len + len(printer_name) + 1  # 1 is space
                state_line = line[state_line_starts:]

                printer = {
                    'name': printer_name,
                    'status': self._parse_status(state_line)
                }
                self.printers.append(printer)

    def _parse_status(self, state_line):
        if 'is idle' in state_line:
            return PRINTER_STATUS_IDLE
        elif 'printing' in state_line:
            return PRINTER_STATUS_PROCESSING
        elif 'disabled' in state_line:
            return PRINTER_STATUS_DISABLED

        return PRINTER_STATUS_UNKNOWN

    def printer_names_by_status(self, status):
        """Gives names of configured printers for a given status

        Arguments:
            status (string)
        """
        names = [prntr['name'] for prntr in self.printers if prntr['status'] == status]
        return names



from insights.parsers.sysconfig import NetworkSysconfig
from insights.tests import context_wrap

NETWORK_SYSCONFIG = """
NETWORKING=yes
HOSTNAME=rhel7-box
GATEWAY=172.31.0.1
NM_BOND_VLAN_ENABLED=no
""".strip()


def test_sysconfig_network():
    result = NetworkSysconfig(context_wrap(NETWORK_SYSCONFIG))
    assert result["GATEWAY"] == '172.31.0.1'
    assert result.get("NETWORKING") == 'yes'
    assert result['NM_BOND_VLAN_ENABLED'] == 'no'



from ...parsers import foreman_rake_db_migrate_status, ParseException
from ...tests import context_wrap
import doctest
import pytest


foreman_rake_db_migrate_status_doc_example = '''
database: foreman

 Status   Migration ID    Migration Name
--------------------------------------------------
   up     20090714132448  Create hosts
   up     20090714132449  Add audits table
   up     20090715143858  Create architectures
   up     20090717025820  Create media
   up     20090718060746  Create domains
   up     20090718064254  Create subnets
   up     20090720134126  Create operatingsystems
   up     20090722140138  Create models
'''

parser = foreman_rake_db_migrate_status.Sat6DBMigrateStatus


def test_FRDMS_doc_examples():
    status = parser(context_wrap(foreman_rake_db_migrate_status_doc_example))
    globs = {
        'Sat6DBMigrateStatus': parser,
        'shared': {parser: status},
        'status': status
    }
    failed, tested = doctest.testmod(foreman_rake_db_migrate_status, globs=globs)
    assert failed == 0


foreman_rake_db_migrate_status_made_up = '''

database: foreman

 Status   Migration ID    Migration Name
--------------------------------------------------
   up     20090714132448  Create hosts
   up     20090714132449  Add audits table
   up     20090715143858  Create architectures
   up     20090717025820  Create media
   up     20090718060746  Create domains
   up     20090718064254  Create subnets
   up     20090720134126  Create operatingsystems
 failed   20090722140138  Create models

'''


def test_FRDMS():
    status = parser(context_wrap(foreman_rake_db_migrate_status_made_up))
    assert status
    assert hasattr(status, 'database')
    assert hasattr(status, 'migrations')
    assert hasattr(status, 'up')
    assert hasattr(status, 'down')

    assert status.database == 'foreman'
    assert len(status.migrations) == 8
    assert '20090714132448' in status.migrations
    assert '20090722140138' in status.migrations
    assert '20090722140137' not in status.migrations
    assert 'up' not in status.migrations

    mig1 = foreman_rake_db_migrate_status.Migration('up', '20090714132448', 'Create hosts')
    assert status.migrations['20090714132448'] == mig1
    mig8 = foreman_rake_db_migrate_status.Migration('failed', '20090722140138', 'Create models')
    assert status.migrations['20090722140138'] == mig8

    assert len(status.up) == 7
    assert status.up[0] == mig1
    assert len(status.down) == 1
    assert status.down[0] == mig8


def test_FRDMS_parse_exception():
    with pytest.raises(ParseException) as exc:
        status = parser(context_wrap(''))
        assert not status
    assert 'Could not find database name nor any migrations' in str(exc)



"""
KatelloServiceStatus - command ``katello-service status``
=========================================================

The KatelloServiceStatus parser only reads the last line of command
``katello-service status`` to get the list of the failed services.

Note:
    Since we only care about the failed services, this parser only process the
    last line which contains the list of the failed services.

The last line of the output of ``katello-service status``::

    Some services failed to status: tomcat6,httpd


Examples:

    >>> kss_ = shared[KatelloServiceStatus]
    >>> kss.is_ok
    False
    >>> kss.failed_services
    ['tomcat6', 'httpd']

"""
from .. import parser, add_filter, CommandParser
from insights.specs import Specs

add_filter(Specs.katello_service_status, ['Some services failed to status',
                                      'Success'])


@parser(Specs.katello_service_status)
class KatelloServiceStatus(CommandParser):
    """
    Read the ``katello-service status`` and get the list of ``failed_services``.

    Attributes:
        failed_services (list): The list of failed services.
        is_ok (bool): Is there no failed service?

    """
    def parse_content(self, content):
        self.failed_services = []
        self.is_ok = False

        for line in content[::-1]:
            if 'Some services failed to status:' in line:
                failed_services = line.split(':')[-1].strip()
                self.failed_services = [s for s in failed_services.split(',') if s]

        self.is_ok = len(self.failed_services) == 0



from insights.core import IniConfigFile, ConfigParser
from insights.tests import context_wrap
from insights.contrib.ConfigParser import NoOptionError
from insights.parsers import SkipException
import pytest

# An example config file with a few tricks and traps for the parser
CONFIG_FILE = """
[global]
keynospace=valuenospaces
key with spaces = value with spaces
key with continued value = value1
                           value2
[comment tricks]
; semicolon comment = should not be found
# hash comment = should not be found
comment # in key = value still found
comment in value = value includes # sign

[value overwriting]
key = value1
key = value2
key = value3
key = this one should be picked

#[commented section]
#this key = should not be found either

[value checks]
positive integer value = 14
negative integer value = -993
positive float value = 3.791
negative float value = -91.2e6
true boolean value = yes
false boolean value = off
""".strip()


def test_ini_config_file_parser():
    ini = IniConfigFile(context_wrap(CONFIG_FILE))

    # sections() tests
    assert list(ini.sections()) == \
        ['global', 'comment tricks', 'value overwriting', 'value checks']

    # items() tests - here we return a dictionary
    assert dict(ini.items('global')) == \
        {'keynospace': 'valuenospaces',
         'key with spaces': 'value with spaces',
         'key with continued value': "value1\nvalue2"}
    assert dict(ini.items('comment tricks')) == \
        {'comment # in key': 'value still found',
         'comment in value': 'value includes # sign'}
    assert dict(ini.items('value overwriting')) == \
        {'key': 'this one should be picked'}

    # get() tests on global section
    assert ini.get('global', 'keynospace') == 'valuenospaces'
    assert ini.get('global', 'key with spaces') == 'value with spaces'
    assert ini.get('global', 'key with continued value') == "value1\nvalue2"
    # keys should not appear in other sections, raise NoOptionError
    with pytest.raises(NoOptionError):
        assert ini.get('global', 'key') is None

    # Other comment tricks
    assert ini.get('comment tricks', 'comment # in key') == 'value still found'
    assert ini.get('comment tricks', 'comment in value') == \
        'value includes # sign'

    # Multiple lines giving the same key - last value overwrites.
    assert ini.get('value overwriting', 'key') == 'this one should be picked'

    # getint / getfloat / getboolean tests
    assert ini.getint('value checks', 'positive integer value') == 14
    assert ini.getint('value checks', 'negative integer value') == -993
    assert ini.getfloat('value checks', 'positive float value') == 3.791
    assert ini.getfloat('value checks', 'negative float value') == -91.2e6
    assert ini.getboolean('value checks', 'true boolean value')
    assert not ini.getboolean('value checks', 'false boolean value')

    # positive has_option tests
    assert ini.has_option('global', 'key with spaces')
    assert ini.has_option('comment tricks', 'comment in value')

    # Negative has_option tests:
    # Commented keys
    with pytest.raises(NoOptionError):
        assert ini.get('comment tricks', 'semicolon comment') is None
    assert not ini.has_option('comment tricks', 'semicolon comment')
    with pytest.raises(NoOptionError):
        assert ini.get('comment tricks', 'hash comment') is None
    assert not ini.has_option('comment tricks', 'hash comment')
    # Commented section
    assert not ini.has_option('commented section', 'this key')

    # __contains__ tests
    assert 'global' in ini
    assert 'value checks' in ini


def test_config_parser_empty():
    with pytest.raises(SkipException):
        assert ConfigParser(context_wrap('')) is None

    with pytest.raises(SkipException):
        assert IniConfigFile(context_wrap('')) is None



"""
NfnetLinkQueue - file ``/proc/net/netfilter/nfnetlink_queue``
=============================================================

Reads the ``/proc/net/netfilter/nfnetlink_queue`` file and creates a list of
dictionaries, one dictionary per row in the file.

The keys of the dictionary are (see
https://home.regit.org/netfilter-en/using-nfqueue-and-libnetfilter_queue/):

- ``queue_number``
- ``peer_portid``: good chance it is process ID of software listening to
  the queue
- ``queue_total``: current number of packets waiting in the queue
- ``copy_mode``: 0 and 1 only message only provide meta data. If 2, the
  message provides a part of packet of size copy range.
- ``copy_range``: length of packet data to put in message
- ``queue_dropped``: number of packets dropped because queue was full
- ``user_dropped``: number of packets dropped because netlink message
  could not be sent to userspace. If this counter is not zero, try
  to increase netlink buffer size. On the application side, you will
  see gap in packet id if netlink message are lost.
- ``id_sequence``: packet id of last packet
- The last field is always '1' and is ignored.

Example Input::

    0  -4423     0 2 65535     0     0       22  1
    1  -4424     0 2 65535     0     0       27  1

Examples:

    >>> # Set up of the environment - ignore this bit:
    >>> nfnetlink_queue_data = '''
    ... 0  -4423     0 2 65535     0     0       22  1
    ... 1  -4424     0 2 65535     0     0       27  1
    ... '''
    >>> from insights.tests import context_wrap
    >>> from insights.parsers.nfnetlink_queue import NfnetLinkQueue
    >>> nf = NfnetLinkQueue(context_wrap(nfnetlink_queue_data))

    >>> # Usual usage in a rule, using the argument name `nf`
    >>> 'copy_mode' in nf.data[0]
    True
    >>> nf.data[0]['copy_mode']  # Note: values as integers
    2

"""
from .. import Parser, parser
from insights.specs import Specs


@parser(Specs.nfnetlink_queue)
class NfnetLinkQueue(Parser):
    """Reads the ``/proc/net/netfilter/nfnetlink_queue`` file and
    creates a list of dictionaries, one dictionary per row in the file.
    """

    def parse_content(self, content):
        self.data = []
        fields = ['queue_number', 'peer_portid', 'queue_total', 'copy_mode',
            'copy_range', 'queue_dropped', 'user_dropped', 'id_sequence']
        for line in content:
            parts = [int(p.strip()) for p in line.split()]
            assert len(parts) == 9
            self.data.append(dict(zip(fields, parts)))



import doctest

from insights.tests import context_wrap
from insights.parsers import dracut_modules
from insights.parsers.dracut_modules import DracutModuleKdumpCaptureService

KDUMP_CAPTURE_SERVICE = """
[Unit]
Description=Kdump Vmcore Save Service
After=dracut-initqueue.service dracut-pre-mount.service dracut-mount.service dracut-pre-pivot.service
Before=initrd-cleanup.service
ConditionPathExists=/etc/initrd-release
OnFailure=emergency.target
OnFailureIsolate=yes

[Service]
Environment=DRACUT_SYSTEMD=1
Environment=NEWROOT=/sysroot
Type=oneshot
ExecStart=/bin/kdump.sh
StandardInput=null
StandardOutput=syslog
StandardError=syslog+console
KillMode=process
RemainAfterExit=yes

# Bash ignores SIGTERM, so we send SIGHUP instead, to ensure that bash
# terminates cleanly.
KillSignal=SIGHUP
""".strip()


def test_dracut_kdump_capture():
    kdump_service_conf = DracutModuleKdumpCaptureService(context_wrap(KDUMP_CAPTURE_SERVICE))
    assert 'Unit' in kdump_service_conf.sections()
    assert 'dracut-initqueue.service' in kdump_service_conf.get('Unit', 'After')


def test_doc():
    failed_count, tests = doctest.testmod(
        dracut_modules,
        globs={
            'config': dracut_modules.DracutModuleKdumpCaptureService(context_wrap(KDUMP_CAPTURE_SERVICE)),
        }
    )
    assert failed_count == 0



from insights.parsers import nova_user_ids
from insights.parsers import ParseException, SkipException
from insights.tests import context_wrap

import doctest
import pytest


NOVA_UID = '''
162
'''.strip()

NOVA_USER_NOT_FOUND = '''
id: nova: no such user
'''.strip()

NOVA_MIGRATION_UID = '''
153
'''.strip()

NOVA_MIGRATION_USER_NOT_FOUND = '''
id: nova_migration: no such user
'''.strip()

UNEXPECTED_OUTPUT = '''
foo
bar
9
'''.strip()


def test_nova_uid():
    nova_uid = nova_user_ids.NovaUID(context_wrap(NOVA_UID))
    assert nova_uid.data == 162

    # 'nova' user not found
    with pytest.raises(SkipException) as ex:
        nova_user_ids.NovaUID(context_wrap(NOVA_USER_NOT_FOUND))
    assert '' in str(ex)

    # Blank input
    with pytest.raises(SkipException) as ex:
        nova_user_ids.NovaUID(context_wrap(''))
    assert '' in str(ex)

    # Any other output. This is not expected
    with pytest.raises(ParseException) as ex:
        nova_user_ids.NovaUID(context_wrap(UNEXPECTED_OUTPUT))
    assert 'Unable to parse user ID' in str(ex)


def test_nova_migration_uid():
    nova_migration_uid = nova_user_ids.NovaMigrationUID(context_wrap(NOVA_MIGRATION_UID))
    assert nova_migration_uid.data == 153

    # 'nova_migration' user not found
    with pytest.raises(SkipException) as ex:
        nova_user_ids.NovaMigrationUID(context_wrap(NOVA_MIGRATION_USER_NOT_FOUND))
    assert '' in str(ex)

    # Blank input
    with pytest.raises(SkipException) as ex:
        nova_user_ids.NovaMigrationUID(context_wrap(''))
    assert '' in str(ex)

    # Any other output. This is not expected
    with pytest.raises(ParseException) as ex:
        nova_user_ids.NovaMigrationUID(context_wrap(UNEXPECTED_OUTPUT))
    assert 'Unable to parse user ID' in str(ex)


def test_doc_examples():
    failed, total = doctest.testmod(
        nova_user_ids,
        globs={'nova_uid': nova_user_ids.NovaUID(context_wrap(NOVA_UID)),
               'nova_migration_uid': nova_user_ids.NovaMigrationUID(context_wrap(NOVA_MIGRATION_UID))}
    )
    assert failed == 0



"""
CloudInitLog - file ``/var/log/cloud-init.log``
-----------------------------------------------
"""
from insights.specs import Specs
from insights import LogFileOutput, parser


@parser(Specs.cloud_init_log)
class CloudInitLog(LogFileOutput):
    """
    Parse the ``/var/log/cloud-init.log`` log file.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`

    Sample input::

        2019-08-07 14:33:27,269 - util.py[DEBUG]: Reading from /etc/cloud/cloud.cfg.d/99-datasource.cfg (quiet=False)
        2019-08-07 14:33:27,269 - util.py[DEBUG]: Read 59 bytes from /etc/cloud/cloud.cfg.d/99-datasource.cfg
        2019-08-07 14:33:27,269 - util.py[DEBUG]: Attempting to load yaml from string of length 59 with allowed root types (<type 'dict'>,)
        2019-08-07 14:33:27,270 - util.py[WARNING]: Failed loading yaml blob. Invalid format at line 1 column 1: "while parsing a block mapping

    Examples:

        >>> "Reading from /etc/cloud/cloud.cfg.d/99-datasource.cfg" in log
        True
        >>> len(log.get('DEBUG')) == 3
        True
    """
    pass



"""
CmdLine - file ``/proc/cmdline``
================================

This parser reads the ``/proc/cmdline`` file, which contains the arguments
given to the currently running kernel on boot.

"""

from insights import Parser, parser, LegacyItemAccess
from insights.parsers import SkipException, ParseException
from insights.specs import Specs


@parser(Specs.cmdline)
class CmdLine(LegacyItemAccess, Parser):
    """
    A parser class for parsing the Linux kernel command line as given in
    ``/proc/cmdline``.

    Parsing Logic::

        Parses all elements in command line to a dict where the key is the
        element itself and the value is a list stores its corresponding values.

        If an element doesn't contain "=", set the corresponding value to `True`.

        If an element contains "=", set the corresponding value to the whole right
        value of the "=".

    .. note::

        For special command line elements that include two "=", e.g.
        ``root=LABEL=/1``, "root" will be the key and "LABEL=/1" will be
        the value in the returned list.

        Some parameters (the returned keys) might be still effective
        even if there is '#' before it, e.g.: ``#rhgb``.  This should
        be checked by the rule.

    Sample input::

        BOOT_IMAGE=/vmlinuz-3.10.0-327.36.3.el7.x86_64 root=/dev/system_vg/Root ro rd.lvm.lv=system_vg/Root crashkernel=auto rd.lvm.lv=system_vg/Swap rhgb quiet LANG=en_GB.utf8

    Examples:
        >>> cmd['BOOT_IMAGE']
        ['/vmlinuz-3.10.0-327.36.3.el7.x86_64']
        >>> cmd['rd.lvm.lv']
        ['system_vg/Root', 'system_vg/Swap']
        >>> 'autofs' in cmd
        False
        >>> cmd.get('autofs') is None
        True
        >>> 'quiet' in cmd
        True
        >>> cmd.get('quiet')
        [True]
        >>> cmd['crashkernel']
        ['auto']

    Attributes:
        data (dict): Parsed booting arguments are stored in this dictionary
        cmdline (str): The RAW line of the ``/proc/cmdline``
    """

    def parse_content(self, content):
        if not content:
            raise SkipException('Empty output')
        if len(content) != 1:
            raise ParseException('Invalid output: {0}', content)

        self.data = {}
        self.cmdline = content[0].strip()
        for el in self.cmdline.split():
            # Thanks to strip and split, el always contains something.
            key, value = el, True
            if "=" in el:
                key, value = el.split("=", 1)
            if key not in self.data:
                self.data[key] = []
            self.data[key].append(value)



"""
TunedConfIni - file ``/etc/tuned.conf``
=======================================
"""

from insights import IniConfigFile, parser, get_active_lines
from insights.specs import Specs


@parser(Specs.tuned_conf)
class TunedConfIni(IniConfigFile):
    """This class parses the ``/etc/tuned.conf`` file using the
    ``IniConfigFile`` base parser.

    Sample configuration file::

        #
        # Net tuning section
        #
        [NetTuning]
        # Enabled or disable the plugin. Default is True. Any other value
        # disables it.
        enabled=False

        #
        # CPU monitoring section
        #
        [CPUMonitor]
        # Enabled or disable the plugin. Default is True. Any other value
        # disables it.
        # enabled=False


    Examples:
        >>> 'NetTuning' in tuned_obj.sections()
        True
        >>> tuned_obj.get('NetTuning', 'enabled') == "False"
        True
        >>> tuned_obj.getboolean('NetTuning', 'enabled') == False
        True
        >>> sorted(tuned_obj.sections())==sorted(['CPUMonitor', 'NetTuning'])
        True
    """

    def parse_content(self, content, allow_no_value=True):
        content = get_active_lines(content)
        super(TunedConfIni, self).parse_content(content, allow_no_value)



from ..ipv6 import IPv6
from ...parsers.modprobe import ModProbe
from ...parsers.lsmod import LsMod
from ...parsers.cmdline import CmdLine
from ...parsers.sysctl import Sysctl
from ...parsers.uname import Uname
from ...tests import context_wrap

from collections import namedtuple

Case = namedtuple('Case', ['cmdline', 'lsmod', 'modprobe', 'sysctl'])

UNAME_RHEL7 = '''
Linux localhost.localdomain 3.10.0-514.6.1.el7.x86_64 #1 SMP Sat Dec 10 11:15:38 EST 2016 x86_64 x86_64 x86_64 GNU/Linux
'''  # noqa

UNAME_RHEL6 = '''
Linux localhost.localdomain 2.6.32-642.el6.x86_64 #1 SMP Wed Apr 13 00:51:26 EDT 2016 x86_64 x86_64 x86_64 GNU/Linux
'''  # noqa

CMDLINE_DISABLED = '''
BOOT_IMAGE=/vmlinuz-3.10.0-514.6.1.el7.x86_64 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap ipv6.disable=1
'''  # noqa

CMDLINE_NOT_DISABLED = '''
BOOT_IMAGE=/vmlinuz-3.10.0-514.6.1.el7.x86_64 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap
'''  # noqa

CMDLINE_RHEL6_DISABLED = '''
ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16  rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet ipv6.disable=1
'''  # noqa

LSMOD_LOADED = '''
Module                  Size  Used by
vboxsf                 37955  1 
ipv6                  336282  12 
i2c_piix4              11232  0 
i2c_core               29132  1 i2c_piix4
snd_intel8x0           30524  0 
'''  # noqa

LSMOD_NOT_LOADED = '''
Module                  Size  Used by
vboxsf                 37955  1 
i2c_piix4              11232  0 
i2c_core               29132  1 i2c_piix4
snd_intel8x0           30524  0 
'''  # noqa

MODPROBE_NOT_DISABLED = '''
# framebuffer drivers
blacklist aty128fb
blacklist atyfb
blacklist radeonfb
blacklist i810fb
blacklist cirrusfb
blacklist intelfb
'''  # noqa

MODPROBE_DISABLED = '''
blacklist radeonfb
blacklist i810fb
blacklist cirrusfb
blacklist intelfb
options ipv6 disable=1
'''  # noqa

MODPROBE_FAKE = '''
blacklist radeonfb
blacklist i810fb
blacklist cirrusfb
blacklist intelfb
install ipv6 /bin/true
install blarfl /bin/true
'''  # noqa

MODPROBE_FAKE_COMMENTED = '''
blacklist radeonfb
blacklist i810fb
blacklist cirrusfb
blacklist intelfb
# install ipv6 /bin/true
'''  # noqa

SYSCTL_DISABLED = '''
net.ipv6.conf.all.disable_ipv6 = 1
'''  # noqa

SYSCTL_NOT_DISABLED = '''
net.ipv6.route.gc_elasticity = 9
net.ipv6.route.gc_interval = 30
net.ipv6.route.gc_min_interval = 0
net.ipv6.route.gc_min_interval_ms = 500
net.ipv6.route.gc_thresh = 1024
net.ipv6.route.gc_timeout = 60
'''  # noqa

CASES = [
        # noqa
        # RHEL7 not disabled
        (7, Case(CMDLINE_NOT_DISABLED, None, None, SYSCTL_NOT_DISABLED),
            (False, set())),
        # RHEL7 disabled via cmdline
        (7, Case(CMDLINE_DISABLED, None, None, SYSCTL_NOT_DISABLED),
            (True, set(['cmdline']))),
        # RHEL7 disabled via sysctl
        (7, Case(CMDLINE_NOT_DISABLED, None, None, SYSCTL_DISABLED),
            (True, set(['sysctl']))),
        # RHEL7 disabled by both cmdline and sysctl
        (7, Case(CMDLINE_DISABLED, None, None, SYSCTL_DISABLED),
            (True, set(['cmdline', 'sysctl']))),
        # RHEL7 with only uname provided
        (7, Case(None, None, None, None),
            (False, set())),
        # RHEL6 loaded not disabled
        (6, Case(None, LSMOD_LOADED, MODPROBE_NOT_DISABLED,
                 SYSCTL_NOT_DISABLED), (False, set())),
        # RHEL6 not loaded but not disabled
        (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_NOT_DISABLED,
                 SYSCTL_NOT_DISABLED), (False, set())),
        # RHEL6 fake installed but loaded
        (6, Case(None, LSMOD_LOADED, MODPROBE_FAKE, SYSCTL_NOT_DISABLED),
            (False, set())),
        # RHEL6 not loaded, fake install commented
        (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_FAKE_COMMENTED,
                 SYSCTL_NOT_DISABLED), (False, set())),
        # RHEL6 loaded but disabled via modprobe
        (6, Case(None, LSMOD_LOADED, MODPROBE_DISABLED, SYSCTL_NOT_DISABLED),
            (True, set(['modprobe_disable']))),
        # RHEL6 not loaded, disabled via modprobe
        (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_DISABLED,
                 SYSCTL_NOT_DISABLED), (True, set(['modprobe_disable']))),
        # RHEL6 not loaded, disabled via fake install
        (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_FAKE, SYSCTL_NOT_DISABLED),
            (True, set(['fake_install']))),
        # RHEL6 loaded but disabled by sysctl
        (6, Case(None, LSMOD_LOADED, MODPROBE_NOT_DISABLED, SYSCTL_DISABLED),
            (True, set(['sysctl']))),
        # RHEL6 not loaded, disabled by sysctl
        (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_NOT_DISABLED,
                 SYSCTL_DISABLED), (True, set(['sysctl']))),
        # RHEL6 disabled by modprobe and sysctl
        (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_DISABLED, SYSCTL_DISABLED),
            (True, set(['sysctl', 'modprobe_disable']))),
        # RHEL6 with lsmod but no modprobe
        (6, Case(None, LSMOD_LOADED, None, None),
            (False, set())),
        # RHEL6 with modprobe but no lsmod
        (6, Case(None, None, MODPROBE_DISABLED, None),
            (True, set(['modprobe_disable']))),
        # RHEL6 with fake install but no lsmod (insufficient data)
        (6, Case(None, None, MODPROBE_FAKE, None),
            (False, set())),
        # RHEL6 with command line only
        (6, Case(CMDLINE_RHEL6_DISABLED, None, None, None),
            (True, set(['cmdline']))),
        ]


def test_integration():
    for rhel, case, result in CASES:
        context = {}
        context[Uname] = Uname(context_wrap(
                               UNAME_RHEL7 if rhel == 7 else UNAME_RHEL6))

        if case.modprobe is not None:
            context[ModProbe] = ModProbe(context_wrap(case.modprobe,
                                         path='/etc/modprobe.d/ipv6.conf'))
        if case.lsmod is not None:
            context[LsMod] = LsMod(context_wrap(case.lsmod))

        if case.cmdline is not None:
            context[CmdLine] = CmdLine(context_wrap(case.cmdline))

        if case.sysctl is not None:
            context[Sysctl] = Sysctl(context_wrap(case.sysctl))

        un = context.get(Uname)
        mp = context.get(ModProbe)
        lsm = context.get(LsMod)
        cl = context.get(CmdLine)
        sct = context.get(Sysctl)

        ipv6 = IPv6(un, mp, lsm, cl, sct)
        assert ipv6.disabled() == result[0]
        assert ipv6.disabled_by() == result[1]



"""
DnfModules - files under in the ``/etc/dnf/modules.d/`` directory
=================================================================

Modularity configuration
"""
from insights import IniConfigFile, parser
from insights.specs import Specs


@parser(Specs.dnf_modules)
class DnfModules(IniConfigFile):
    """
    Provides access to state of enabled modules/streams/profiles
    which is located in the /etc/dnf/modules.d/ directory

    Examples:
        >>> len(dnf_modules.sections())
        3
        >>> str(dnf_modules.get("postgresql", "stream"))
        '9.6'
        >>> str(dnf_modules.get("postgresql", "profiles"))
        'client'
    """
    pass



import pytest
from six import StringIO
from insights import dr, make_fail, rule
from insights.formats.text import HumanReadableFormat
from insights.formats._yaml import YamlFormat
from insights.formats._json import JsonFormat
from insights.formats._syslog import SysLogFormat
from insights.formats.html import HtmlFormat
from insights.formats.simple_html import SimpleHtmlFormat


SL_MSG = "Running insights.tests.test_formats.report"
SL_CMD = "Command Line - "
SL_ARCHIVE = "Real Archive Path - "
SL_PATH = "/insights/core"


@rule()
def report():
    return make_fail("ERROR", foo="bar")


def test_human_readable():
    broker = dr.Broker()
    output = StringIO()
    with HumanReadableFormat(broker, stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()
    assert "foo" in data
    assert "bar" in data


def test_json_format():
    broker = dr.Broker()
    output = StringIO()
    with JsonFormat(broker, stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()
    assert "foo" in data
    assert "bar" in data


@pytest.mark.syslog_format
def test_syslog_format_no_archive():
    broker = dr.Broker()
    output = StringIO()
    with SysLogFormat(broker, stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()
    assert SL_MSG in data
    assert SL_CMD in data


@pytest.mark.syslog_format
def test_syslog_format_archive():
    broker = dr.Broker()
    output = StringIO()
    with SysLogFormat(broker, archive="../../insights/core", stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()

    assert SL_MSG in data
    assert SL_CMD in data
    assert SL_ARCHIVE in data
    assert SL_PATH in data


def test_yaml_format():
    broker = dr.Broker()
    output = StringIO()
    with YamlFormat(broker, stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()
    assert "foo" in data
    assert "bar" in data


def test_html_format():
    broker = dr.Broker()
    output = StringIO()
    with HtmlFormat(broker, stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()
    assert "foo" in data
    assert "bar" in data


def test_simple_html_format():
    broker = dr.Broker()
    output = StringIO()
    with SimpleHtmlFormat(broker, stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()
    assert "foo" in data
    assert "bar" in data



from .. import parser, LegacyItemAccess, CommandParser
from insights.specs import Specs


@parser(Specs.rhn_charsets)
class RHNCharSets(LegacyItemAccess, CommandParser):
    """
    ==== Sample (1) embedded database ====
     server_encoding
    -----------------
     UTF8
    (1 row)

     client_encoding
    -----------------
     UTF8
    (1 row)
    ==== Sample (2) Oracle database ====
    PARAMETER                  VALUE
    ---------------------------------
    NLS_CHARACTERSET           UTF8
    NLS_NCHAR_CHARACTERSET     UTF8
    ======================================
    Returns a dict:
    - {'server_encoding': 'UTF8','client_encoding': 'UTF8'}
    - {'NLS_CHARACTERSET': 'UTF8','NLS_NCHAR_CHARACTERSET': 'UTF8'}
    """

    def parse_content(self, content):
        db_set = {}
        db_backend = None
        in_server = False
        in_client = False
        for line in content:
            line = line.strip()
            # skip empty and useless lines
            if not line or line.startswith(('----', '(', 'PARAMETER')):
                continue
            if '_encoding' in line:
                db_backend = 'postgresql'
                in_server = line.startswith('server_')
                in_client = line.startswith('client_')
            elif db_backend == 'postgresql':
                if in_server:
                    db_set['server_encoding'] = line
                elif in_client:
                    db_set['client_encoding'] = line
            elif line.startswith('NLS_'):
                line_splits = line.split()
                if len(line_splits) == 2:
                    db_set[line_splits[0]] = line_splits[1]
        self.data = db_set



from .. import parser, get_active_lines, Parser
import re
from insights.specs import Specs


@parser(Specs.scheduler)
class Scheduler(Parser):

    def parse_content(self, content):
        active_scheduler_regex = re.compile(r'\[.*]')
        result = {}
        for line in get_active_lines(content):
            for sched in line.split():
                active_scheduler = active_scheduler_regex.search(sched)
                if active_scheduler:
                    result[self.file_path.split('/')[3]] = active_scheduler.group()
        self.data = result



"""
Ovirt Engine logs
=================

This module contains the following parsers:

ServerLog - file ``/var/log/ovirt-engine/server.log``
-----------------------------------------------------
UILog - file ``/var/log/ovirt-engine/ui.log``
---------------------------------------------
EngineLog - file ``/var/log/ovirt-engine/engine.log``
-----------------------------------------------------
BootLog - file ``/var/log/ovirt-engine/boot.log``
-------------------------------------------------
ConsoleLog - file ``/var/log/ovirt-engine/console.log``
-------------------------------------------------------
"""
from time import strptime
from datetime import datetime
from insights import LogFileOutput, Syslog, parser
from insights.specs import Specs


@parser(Specs.ovirt_engine_boot_log)
class BootLog(Syslog):
    '''Provide access to ``/var/log/ovirt-engine/boot.log`` using the Syslog parser class.

    Sample input::

        03:46:19,238 INFO  [org.jboss.as.server] WFLYSRV0039: Creating http management service using socket-binding (management)
        03:46:19,242 INFO  [org.xnio] XNIO version 3.5.5.Final-redhat-1
        03:46:19,250 INFO  [org.xnio.nio] XNIO NIO Implementation Version 3.5.5.Final-redhat-1

    Examples:

        >>> xnio_lines = boot_log.get('xnio.nio')
        >>> len(xnio_lines)
        1
        >>> xnio_lines[0].get('procname')
        'org.xnio.nio'
        >>> xnio_lines[0].get('level')
        'INFO'
        >>> xnio_lines[0].get('message')
        'XNIO NIO Implementation Version 3.5.5.Final-redhat-1'
    '''
    time_format = '%H:%M:%S,%f'

    def _parse_line(self, line):
        msg_info = {'raw_message': line}
        line_splits = line.split()
        try:
            if strptime(line_splits[0], self.time_format):
                msg_info['timestamp'] = strptime(line_splits[0], self.time_format)
                msg_info['level'] = line_splits[1]
                msg_info['procname'] = line_splits[2].strip('[]')
                msg_info['message'] = ' '.join(line_splits[3:])
        except ValueError:
            pass
        return msg_info


@parser(Specs.ovirt_engine_console_log)
class ConsoleLog(LogFileOutput):
    '''Provide access to ``/var/log/ovirt-engine/console.log`` using the LogFileoutput parser class.'''
    pass


# Using existing engine_log specs
@parser(Specs.engine_log)
class EngineLog(LogFileOutput):
    '''Provide access to ``/var/log/ovirt-engine/engine.log`` using the LogFileoutput parser class.

    Sample input::

        2018-08-06 04:06:33,229+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engine' is using 0 threads out of 500, 8 threads waiting for tasks and 0 tasks in queue.
        2018-08-06 04:06:33,229+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engineScheduled' is using 0 threads out of 100, 100 threads waiting for tasks.
        2018-08-06 04:06:33,229+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engineThreadMonitoring' is using 1 threads out of 1, 0 threads waiting for tasks.
        2018-08-06 04:06:33,229+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'hostUpdatesChecker' is using 0 threads out of 5, 5 threads waiting for tasks.
        2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'commandCoordinator' is using 0 threads out of 10, 2 threads waiting for tasks.
        2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'default' is using 0 threads out of 1, 5 threads waiting for tasks.
        2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engine' is using 0 threads out of 500, 8 threads waiting for tasks and 0 tasks in queue.
        2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engineScheduled' is using 0 threads out of 100, 100 threads waiting for tasks.
        2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engineThreadMonitoring' is using 1 threads out of 1, 0 threads waiting for tasks.
        2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'hostUpdatesChecker' is using 0 threads out of 5, 5 threads waiting for tasks.
        2018-08-06 04:26:33,233+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'commandCoordinator' is using 0 threads out of 10, 2 threads waiting for tasks.
        2018-08-06 04:26:33,233+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'default' is using 0 threads out of 1, 5 threads waiting for tasks.
        2018-08-06 04:26:33,233+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engine' is using 0 threads out of 500, 8 threads waiting for tasks and 0 tasks in queue.
        2018-08-22 00:16:14,357+05 INFO  [org.ovirt.engine.core.vdsbroker.vdsbroker.HotUnplugLeaseVDSCommand] (default task-133) [e3bc976c-bc3e-4b41-807f-3a518169ad18] START, HotUnplugLeaseVDSCommand(HostName = example.com, LeaseVDSParameters:{hostId='bfa308ab-5add-4ad7-8f1c-389cb8dcf703', vmId='789489a3-be62-40e4-b13e-beb34ba5ff93'}), log id: 7a634963

    Examples:

        >>> from datetime import datetime
        >>> "Thread pool 'engine'" in engine_log
        True
        >>> len(list(engine_log.get_after(datetime(2018, 8, 6, 4, 16, 33, 0))))
        10
        >>> matched_line = "2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'hostUpdatesChecker' is using 0 threads out of 5, 5 threads waiting for tasks."
        >>> engine_log.get('hostUpdatesChecker')[-1].get('raw_message') == matched_line
        True
        >>> engine_log.get('vdsbroker')[-1].get('procname') == 'org.ovirt.engine.core.vdsbroker.vdsbroker.HotUnplugLeaseVDSCommand'
        True
    '''
    def _parse_line(self, line):
        msg_info = {'raw_message': line}
        line_splits = line.split()
        dt = ' '.join(line_splits[0:2]).split(',')[0]
        try:
            if datetime.strptime(dt, self.time_format):
                msg_info['timestamp'] = datetime.strptime(dt, self.time_format)
                msg_info['level'] = line_splits[2]
                msg_info['procname'] = line_splits[3].strip('[]')
                msg_info['message'] = ' '.join(line_splits[4:])
        except ValueError:
            pass
        return msg_info


@parser(Specs.ovirt_engine_server_log)
class ServerLog(EngineLog):
    '''Provide access to ``/var/log/ovirt-engine/server.log`` using the EngineLog parser class.

    Sample input::

        2018-01-17 01:46:15,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-6) WFLYSRV0207: Starting subdeployment (runtime-name: "services.war")
        2018-01-17 01:46:16,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-1) WFLYSRV0207: Starting subdeployment (runtime-name: "webadmin.war")
        2018-01-17 01:46:17,739+05 WARN  [org.jboss.as.dependency.unsupported] (MSC service thread 1-7) WFLYSRV0019: Deployment "deployment.engine.ear" is using an unsupported module ("org.dom4j") which may be changed or removed in future versions without notice.


    Examples:

        >>> 'is using an unsupported module' in server_log
        True
        >>> from datetime import datetime
        >>> len(list(server_log.get_after(datetime(2018, 1, 17, 1, 46, 16, 0))))
        2
        >>> matched_line = '2018-01-17 01:46:17,739+05 WARN  [org.jboss.as.dependency.unsupported] (MSC service thread 1-7) WFLYSRV0019: Deployment "deployment.engine.ear" is using an unsupported module ("org.dom4j") which may be changed or removed in future versions without notice.'
        >>> server_log.get('WARN')[-1].get('raw_message') == matched_line
        True
        >>> sec_lines = server_log.get('org.wildfly.security')
        >>> len(sec_lines)
        1
        >>> sec_lines[0]['level']
        'INFO'
    '''
    pass


@parser(Specs.ovirt_engine_ui_log)
class UILog(EngineLog):
    '''Provide access to ``/var/log/ovirt-engine/ui.log`` using the EngineLog parser class.

    Sample input::

        2018-01-24 05:31:26,243+05 ERROR [org.ovirt.engine.ui.frontend.server.gwt.OvirtRemoteLoggingService] (default task-134) [] Permutation name: C068E8B2E40A504D3054A1BDCF2A72BB
        2018-01-24 05:32:26,243+05 ERROR [org.ovirt.engine.ui.frontend.server.gwt.OvirtRemoteLoggingService] (default task-134) [] Uncaught exception: com.google.gwt.core.client.JavaScriptException: (TypeError)

    Examples:

        >>> 'Permutation name' in ui_log
        True
        >>> from datetime import datetime
        >>> len(list(ui_log.get_after(datetime(2018, 1, 24, 5, 31, 26, 0))))
        2
        >>> exception_lines = ui_log.get('Uncaught exception')
        >>> len(exception_lines)
        1
        >>> exception_lines[0].get('procname')
        'org.ovirt.engine.ui.frontend.server.gwt.OvirtRemoteLoggingService'
        >>> exception_lines[0].get('level')
        'ERROR'
    '''
    pass



"""
SCSIEhDead - file ``/sys/class/scsi_host/host[0-9]*/eh_deadline``
=================================================================

This parser parses the content from eh_deadline file from
individual SCSI hosts. This parser will return data in
dictionary format.

Sample content from ``/sys/class/scsi_host/host0/eh_deadline``::

    off/10/-1/0

Examples:
    >>> type(scsi_obj0)
    <class 'insights.parsers.scsi_eh_deadline.SCSIEhDead'>
    >>> scsi_obj0.data
    {'host0': 'off'}
    >>> scsi_obj0.scsi_host
    'host0'
    >>> type(scsi_obj1)
    <class 'insights.parsers.scsi_eh_deadline.SCSIEhDead'>
    >>> scsi_obj1.data
    {'host1': '10'}
    >>> scsi_obj1.scsi_host
    'host1'
    >>> type(scsi_obj2)
    <class 'insights.parsers.scsi_eh_deadline.SCSIEhDead'>
    >>> scsi_obj2.data
    {'host2': '-1'}
    >>> scsi_obj2.scsi_host
    'host2'

"""

from insights import Parser, parser, get_active_lines, LegacyItemAccess
from insights.specs import Specs


@parser(Specs.scsi_eh_deadline)
class SCSIEhDead(LegacyItemAccess, Parser):
    """
    Parse `/sys/class/scsi_host/host[0-9]*/eh_deadline` file, return a dict
    contain `eh_deadline` scsi host file info. "scsi_host" key is scsi host file
    parse from scsi host file name.

    Properties:
        scsi_host (str): scsi host file name derived from file path.
    """

    def __init__(self, context):
        self.data = {}
        self.scsi_host = context.path.rsplit("/")[-2]
        super(SCSIEhDead, self).__init__(context)

    def parse_content(self, content):
        for line in get_active_lines(content):
            self.data[self.scsi_host] = line

    @property
    def host_eh_deadline(self):
        """
        (list): It will return the scsi host modes when set else `None`.
        """
        return self.data[self.scsi_host]



from insights.parsers.jboss_version import JbossVersion
from insights.tests import context_wrap
import doctest
from insights.parsers import jboss_version

JBOSS_6 = """
Red Hat JBoss Enterprise Application Platform - Version 6.4.3.GA
""".strip()

JBOSS_7 = """
Red Hat JBoss Enterprise Application Platform - Version 7.1.0.Beta1
""".strip()


def test_jboss6():
    release6 = JbossVersion(context_wrap(JBOSS_6, path='/home/test/jboss/jboss-eap-6.4/version.txt'))
    assert release6.file_path == '/home/test/jboss/jboss-eap-6.4/version.txt'
    assert release6.raw == JBOSS_6
    assert release6.major == 6
    assert release6.minor == 4
    assert release6.version == "6.4.3"
    assert release6.code_name == "GA"
    assert release6.release == 3


def test_jboss7():
    release7 = JbossVersion(context_wrap(JBOSS_7, path='/home/test/jboss/jboss-eap-7.1/version.txt'))
    assert release7.file_path == '/home/test/jboss/jboss-eap-7.1/version.txt'
    assert release7.raw == JBOSS_7
    assert release7.major == 7
    assert release7.minor == 1
    assert release7.version == "7.1.0"
    assert release7.code_name == "Beta1"
    assert release7.release == 0


def test_jboss_version_doc_examples():
    env = {
        'JbossVersion': JbossVersion,
        'jboss_version': JbossVersion(context_wrap(JBOSS_6,
                                                   path='/home/test/jboss/jboss-eap-6.4/version.txt'))
    }
    failed, total = doctest.testmod(jboss_version, globs=env)
    assert failed == 0



import doctest
from insights.parsers import ceph_log
from insights.parsers.ceph_log import CephLog
from insights.tests import context_wrap

from datetime import datetime


CEPH_LOG = """
2017-05-31 13:01:44.034376 mon.0 192.xx.xx.xx:6789/0 742585 : cluster [INF] pgmap v5133969: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 2027 kB/s rd, 20215 kB/s wr, 711 op/s
2017-05-31 13:01:45.041760 mon.0 192.xx.xx.xx:6789/0 742586 : cluster [INF] pgmap v5133970: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 1606 kB/s rd, 17354 kB/s wr, 718 op/s
2017-05-31 13:01:46.933829 osd.22 192.xx.xx.xx:6814/42154 172581 : cluster [WRN] 44 slow requests, 2 included below; oldest blocked for > 49.982746 secs
2017-05-31 13:01:46.933946 osd.22 192.xx.xx.xx:6814/42154 172582 : cluster [WRN] slow request 30.602517 seconds old, received at 2017-05-31 13:01:06.330484: osd_op(client.3395798.0:2855671 1.54392173 gnocchi_06c8214c-afae-4e64-8a4a-a466c4f257dc_1244160000.0_median_86400.0_v3 [write 26253~9] snapc 0=[] ondisk+write+known_if_redirected e487) currently waiting for subops from 23
2017-05-31 13:01:46.933955 osd.22 192.xx.xx.xx:6814/42154 172583 : cluster [WRN] slow request 30.530961 seconds old, received at 2017-05-31 13:01:06.402041: osd_op(client.324182.0:46141816 1.e637a4b3 measure [omap-rm-keys 0~107] snapc 0=[] ondisk+write+skiprwlocks+known_if_redirected e487) currently waiting for subops from 23
2017-05-31 13:01:47.050539 mon.0 192.xx.xx.xx:6789/0 742589 : cluster [INF] pgmap v5133971: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 1597 kB/s rd, 7259 kB/s wr, 398 op/s
2017-05-31 13:01:48.057187 mon.0 192.xx.xx.xx:6789/0 742590 : cluster [INF] pgmap v5133972: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 2373 kB/s rd, 5138 kB/s wr, 354 op/s
2017-05-31 13:01:49.064950 mon.0 192.xx.xx.xx:6789/0 742598 : cluster [INF] pgmap v5133973: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 4187 kB/s rd, 10266 kB/s wr, 714 op/s
2017-05-31 13:01:50.069437 mon.0 192.xx.xx.xx:6789/0 742599 : cluster [INF] pgmap v5133974: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 470 MB/s rd, 11461 kB/s wr, 786 op/s
""".strip()


def test_ceph_log():
    ceph_logs = CephLog(context_wrap(CEPH_LOG))
    assert len(ceph_logs.get("[WRN] slow request")) == 2
    assert len(list(ceph_logs.get_after(datetime(2017, 5, 31, 13, 1, 46)))) == 7
    assert len(ceph_logs.get("[INF]")) == 6
    assert "slow requests" in ceph_logs


def test_doc():
    env = {"ceph_log": CephLog(context_wrap(CEPH_LOG, path="/var/log/ceph/ceph.log"))}
    failed, total = doctest.testmod(ceph_log, globs=env)
    assert failed == 0



import doctest
from insights.tests import context_wrap
from insights.combiners import du
from insights.parsers.du import DiskUsageDir

DISK_USAGE_DIR_SAMPLE1 = """
553500	/var/log
""".strip()

DISK_USAGE_DIR_SAMPLE2 = """
519228	/var/lib/pgsql
""".strip()


def test_disk_usage_dirs():
    parser1 = DiskUsageDir(context_wrap(DISK_USAGE_DIR_SAMPLE1))
    parser2 = DiskUsageDir(context_wrap(DISK_USAGE_DIR_SAMPLE2))
    disk_usage_dirs = du.DiskUsageDirs([parser1, parser2])
    assert disk_usage_dirs is not None
    assert set(disk_usage_dirs.keys()) == set(["/var/log", "/var/lib/pgsql"])
    assert disk_usage_dirs["/var/log"] == 553500
    assert disk_usage_dirs["/var/lib/pgsql"] == 519228


def test_disk_usage_dirs_docs():
    parser1 = DiskUsageDir(context_wrap(DISK_USAGE_DIR_SAMPLE1))
    parser2 = DiskUsageDir(context_wrap(DISK_USAGE_DIR_SAMPLE2))
    env = {'disk_usage_dirs': du.DiskUsageDirs([parser1, parser2])}
    failed, total = doctest.testmod(du, globs=env)
    assert failed == 0



"""
Sealert - command ``/usr/bin/sealert -l "*"``
=============================================
"""

from insights import CommandParser
from insights import parser
from insights.specs import Specs
from insights.parsers import SkipException


class Report(object):

    def __init__(self):
        self.lines = []

    def __str__(self):
        return "\n".join(self.lines).strip()

    def append_line(self, x):
        self.lines.append(x)

    def lines_stripped(self):
        """
        Returns the lines without trailing empty lines

        Returns:
            list: The lines without empty lines at the end of the list.
        """
        lines = self.lines[:]
        for index in range(len(self.lines) - 1, -1, -1):
            if lines[index] == '':
                del(lines[index])
            else:
                break
        return lines


@parser(Specs.sealert)
class Sealert(CommandParser):
    """
    Reads the output of ``/usr/bin/sealert -l "*"``.

    Sample output:

    .. code-block:: none

        SELinux is preventing sh from entrypoint access on the file /usr/bin/podman.

        *****  Plugin catchall (100. confidence) suggests **************************

        If you believe that sh should be allowed entrypoint access on the podman file by default.
        Then you should report this as a bug.
        You can generate a local policy module to allow this access.
        Do
        allow this access for now by executing:
        # ausearch -c 'sh' --raw | audit2allow -M my-sh
        # semodule -X 300 -i my-sh.pp

        Additional Information:
        Source Context unconfined_u:system_r:rpm_script_t:s0-s0:c0.c1023
        Target Context system_u:object_r:container_runtime_exec_t:s0
        Target Objects                /usr/bin/podman [ file ]
        Source                        sh
        Source Path                   sh
        Port                          <Unknown>
        Host                          localhost.localdomain
        Source RPM Packages
        Target RPM Packages           podman-1.1.2-1.git0ad9b6b.fc28.x86_64
        Policy RPM                    selinux-policy-3.14.1-54.fc28.noarch
        Selinux Enabled               True
        Policy Type                   targeted
        Enforcing Mode                Enforcing
        Host Name                     localhost.localdomain
        Platform                      Linux localhost.localdomain 4.20.7-100.fc28.x86_64
                                      #1 SMP Wed Feb 6 19:17:09 UTC 2019 x86_64 x86_64
        Alert Count                   1
        First Seen                    2019-07-30 11:15:04 CEST
        Last Seen                     2019-07-30 11:15:04 CEST
        Local ID                      39a7094b-e402-4d87-9af9-e97eda41219a

        Raw Audit Messages
        type=AVC msg=audit(1564478104.911:4631): avc:  denied  { entrypoint } for  pid=29402 comm="sh" path="/usr/bin/podman" dev="dm-1" ino=955465 scontext=unconfined_u:system_r:rpm_script_t:s0-s0:c0.c1023 tcontext=system_u:object_r:container_runtime_exec_t:s0 tclass=file permissive=0

        Hash: sh,rpm_script_t,container_runtime_exec_t,file,entrypoint

    Examples:
        >>> type(sealert)
        <class 'insights.parsers.sealert.Sealert'>
        >>> sealert.raw_lines[0]
        'SELinux is preventing rngd from using the dac_override capability.'
        >>> sealert.reports[1].lines_stripped()[0]
        'SELinux is preventing sh from entrypoint access on the file /usr/bin/podman.'
        >>> str(sealert.reports[1]).split('\\n')[0]
        'SELinux is preventing sh from entrypoint access on the file /usr/bin/podman.'

    Attributes:
        raw_lines (list[str]): Unparsed output as list of lines
        reports (list[Report]): Sealert reports

    Raises:
        SkipException: When output is empty
    """

    def parse_content(self, content):
        if not content:
            raise SkipException("Input content is empty")

        self.raw_lines = content
        self.reports = []

        for line in content:
            if line.startswith("SELinux is preventing "):
                self.reports.append(Report())
            if self.reports:  # skips the first report if it contains only partial data
                self.reports[-1].append_line(line)

        if not self.reports:
            raise SkipException("No sealert reports")



from insights.parsers import lssap, ParseException, SkipException
from insights.tests import context_wrap
from insights.util import keys_in
import doctest
import pytest

Lssap_BAD = """
 - lssap version 1.0 -
 ==========================================
   SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
   HA2|  16|       D16|         lu0417
==========================================
""".strip()

Lssap_BAD1 = """
   HA2|  16|       D16|         lu0417
""".strip()

Lssap_BAD2 = ""

Lssap_nw_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  50|       D50|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D50/exe
  HA2|  51|       D51|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D51/exe
==========================================
""".strip()

Lssap_hana_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       HDB16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
==========================================
""".strip()

Lssap_ascs_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       ASCS16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/ASCS16/exe
==========================================
""".strip()

Lssap_runAll_Test = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  16|     HDB16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
  HA2|  16|    ASCS16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/ASCS16/exe
  DA1|  97|    SMDA97|         l0116a|745, patch 100, changelist 1652052|         /usr/sap/DA1/SMDA97/exe
  DAA|  98|    SMDA98|         l0116a|745, patch 100, changelist 1652052|         /usr/sap/DAA/SMDA98/exe
  MHP|  00|     HDB00|         l0116a|749, patch 315, changelist 1787247|         /usr/sap/MHP/HDB00/exe
  MEP|  02|    ASCS02|         l0116a|745, patch 600, changelist 1791210|         /usr/sap/MEP/ASCS02/exe
  MEP|  01| DVEBMGS01|         l0116a|745, patch 600, changelist 1791210|         /usr/sap/MEP/DVEBMGS01/exe
==========================================
""".strip()

Lssap_BAD_1 = """
/usr/sap/hostctrl/lssap: command or file not found
"""

Lssap_BAD_2 = """
HA2 | N16|  D16
""".strip()

Lssap_BAD_3 = """
HA2 | N16 | D16
HB2 | foo | bar
""".strip()


def test_doc_examples():
    env = {'lssap': lssap.Lssap(context_wrap(Lssap_nw_TEST))}
    failed, total = doctest.testmod(lssap, globs=env)
    assert failed == 0


def test_lssap_netweaver():
    sap = lssap.Lssap(context_wrap(Lssap_nw_TEST))
    data = sap.data
    assert keys_in(["SID", "Nr", "Instance", "SAPLOCALHOST", "Version", "DIR_EXECUTABLE"], data[0])
    assert data[0] == {"SID": "HA2", "Nr": "16", "Instance": "D16", "SAPLOCALHOST": "lu0417",
                       "Version": "749, patch 10, changelist 1698137",
                       "DIR_EXECUTABLE": "/usr/sap/HA2/D16/exe"}
    assert data[2]["Instance"] == "D50"
    assert data[-1]["SAPLOCALHOST"] == "lu0417"
    assert sap.is_netweaver() is True
    assert sap.is_hana() is False
    assert sap.is_ascs() is False


def test_lssap_hana():
    sap = lssap.Lssap(context_wrap(Lssap_hana_TEST))
    data = sap.data
    assert data[0] == {"SID": "HA2", "Nr": "16", "Instance": "HDB16", "SAPLOCALHOST": "lu0417",
                       "Version": "749, patch 10, changelist 1698137",
                       "DIR_EXECUTABLE": "/usr/sap/HA2/HDB16/exe"}
    assert data[0]["SAPLOCALHOST"] == "lu0417"
    assert sap.is_hana() is True
    assert sap.is_netweaver() is False
    assert sap.is_ascs() is False


def test_lssap_ascs():
    sap = lssap.Lssap(context_wrap(Lssap_ascs_TEST))
    assert sap.is_hana() is False
    assert sap.is_netweaver() is False
    assert sap.is_ascs() is True


def test_all():
    sap = lssap.Lssap(context_wrap(Lssap_runAll_Test))
    assert sap.sid == ['DA1', 'DAA', 'HA2', 'MEP', 'MHP']
    assert sap.instances == ['D16', 'D22', 'HDB16', 'ASCS16', 'SMDA97', 'SMDA98', 'HDB00', 'ASCS02', 'DVEBMGS01']
    assert sap.data[1] == {"SID": "HA2", "Nr": "22", "Instance": "D22", "SAPLOCALHOST": "lu0417",
                           "Version": "749, patch 10, changelist 1698137",
                           "DIR_EXECUTABLE": "/usr/sap/HA2/D22/exe"}
    assert sap.is_netweaver() is True
    assert sap.is_hana() is True
    assert sap.is_ascs() is True
    assert 'SMDA' in sap.instance_types
    assert 'DVEBMGS01' in sap.instances
    assert sap.version("D16") == "749, patch 10, changelist 1698137"
    assert sap.version("D06") is None


def test_fail():
    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_BAD_1))
    assert "Lssap: Unable to parse 1 line(s) of content: (['/usr/sap/hostctrl/lssap: command or file not found'])" in str(excinfo)

    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_BAD_2))
    assert "Lssap: Unable to parse 1 line(s) of content: (['HA2 | N16|  D16'])" in str(excinfo)

    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_BAD_3))
    assert "Lssap: Unable to parse 2 line(s) of content: (['HA2 | N16 | D16', 'HB2 | foo | bar'])" in str(excinfo)

    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_BAD1))
    assert "Lssap: Unable to parse 1 line(s) of content: (['HA2|  16|       D16|         lu0417'])" in str(excinfo)

    with pytest.raises(SkipException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_BAD2))

    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap('test'))
    assert "Lssap: Unable to parse 1 line(s) of content: (['test'])" in str(excinfo)



from insights.parsers.pcs_status import PCSStatus
from insights.tests import context_wrap

CLUSTER_NORMAL = """
Cluster name: openstack
Last updated: Fri Oct 14 15:45:32 2016
Last change: Thu Oct 13 20:02:27 2016
Stack: corosync
Current DC: myhost15 (1) - partition with quorum
Version: 1.1.12-a14efad
3 Nodes configured
143 Resources configured


Online: [ myhost15 myhost16 myhost17 ]

Full list of resources:

 stonith-ipmilan-10.24.221.172	(stonith:fence_ipmilan):	Started myhost15
 stonith-ipmilan-10.24.221.171	(stonith:fence_ipmilan):	Started myhost16
 stonith-ipmilan-10.24.221.173	(stonith:fence_ipmilan):	Started myhost15
 ip-ceilometer-pub-10.50.218.121	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-neutron-pub-10.50.218.129	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-ceilometer-prv-10.24.82.127	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-ceilometer-adm-10.24.82.126	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-horizon-pub-10.50.218.126	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-horizon-adm-10.24.82.137	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-amqp-pub-10.24.82.145	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-loadbalancer-pub-10.50.218.128	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-neutron-adm-10.24.82.141	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-neutron-prv-10.24.82.142	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-horizon-prv-10.24.82.138	(ocf::heartbeat:IPaddr2):	Started myhost16
 Clone Set: memcached-clone [memcached]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: haproxy-clone [haproxy]
     Started: [ myhost15 myhost16 myhost17 ]
 ip-galera-pub-10.24.82.130	(ocf::heartbeat:IPaddr2):	Started myhost15
 Master/Slave Set: galera-master [galera]
     Masters: [ myhost15 myhost16 myhost17 ]
 Clone Set: rabbitmq-server-clone [rabbitmq-server]
     Started: [ myhost15 myhost16 myhost17 ]
 ip-keystone-pub-10.50.218.127	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-keystone-adm-10.24.82.139	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-keystone-prv-10.24.82.140	(ocf::heartbeat:IPaddr2):	Started myhost15
 Clone Set: keystone-clone [keystone]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: fs-varlibglanceimages-clone [fs-varlibglanceimages]
     Started: [ myhost15 myhost16 myhost17 ]
 ip-glance-pub-10.50.218.123	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-glance-prv-10.24.82.132	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-glance-adm-10.24.82.131	(ocf::heartbeat:IPaddr2):	Started myhost16
 Clone Set: glance-registry-clone [glance-registry]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: glance-api-clone [glance-api]
     Started: [ myhost15 myhost16 myhost17 ]
 ip-nova-pub-10.50.218.130	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-nova-adm-10.24.82.143	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-nova-prv-10.24.82.144	(ocf::heartbeat:IPaddr2):	Started myhost15
 Clone Set: openstack-nova-novncproxy-clone [openstack-nova-novncproxy]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: openstack-nova-consoleauth-clone [openstack-nova-consoleauth]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: openstack-nova-conductor-clone [openstack-nova-conductor]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: openstack-nova-api-clone [openstack-nova-api]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: openstack-nova-scheduler-clone [openstack-nova-scheduler]
     Started: [ myhost15 myhost16 myhost17 ]
 ip-cinder-pub-10.50.218.122	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-cinder-prv-10.24.82.129	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-cinder-adm-10.24.82.128	(ocf::heartbeat:IPaddr2):	Started myhost16
 Clone Set: cinder-api-clone [cinder-api]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: cinder-scheduler-clone [cinder-scheduler]
     Started: [ myhost15 myhost16 myhost17 ]
 cinder-volume	(systemd:openstack-cinder-volume):	Started myhost15
 ip-heat-pub-10.50.218.124	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-heat-prv-10.24.82.134	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-heat-adm-10.24.82.133	(ocf::heartbeat:IPaddr2):	Started myhost15
 ip-heat_cfn-pub-10.50.218.125	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-heat_cfn-prv-10.24.82.136	(ocf::heartbeat:IPaddr2):	Started myhost16
 ip-heat_cfn-adm-10.24.82.135	(ocf::heartbeat:IPaddr2):	Started myhost16
 Clone Set: neutron-server-clone [neutron-server]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: neutron-scale-clone [neutron-scale] (unique)
     neutron-scale:0	(ocf::neutron:NeutronScale):	Started myhost17
     neutron-scale:1	(ocf::neutron:NeutronScale):	Started myhost16
     neutron-scale:2	(ocf::neutron:NeutronScale):	Started myhost15
 Clone Set: neutron-ovs-cleanup-clone [neutron-ovs-cleanup]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: neutron-netns-cleanup-clone [neutron-netns-cleanup]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: neutron-openvswitch-agent-clone [neutron-openvswitch-agent]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: neutron-dhcp-agent-clone [neutron-dhcp-agent]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: neutron-l3-agent-clone [neutron-l3-agent]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: neutron-metadata-agent-clone [neutron-metadata-agent]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: heat-api-clone [heat-api]
     Started: [ myhost15 myhost16 myhost17 ]
 Resource Group: heat
     openstack-heat-engine	(systemd:openstack-heat-engine):	Started myhost15
 Clone Set: heat-api-cfn-clone [heat-api-cfn]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: heat-api-cloudwatch-clone [heat-api-cloudwatch]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: horizon-clone [horizon]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: mongod-clone [mongod]
     Started: [ myhost15 myhost16 myhost17 ]
 openstack-ceilometer-central	(systemd:openstack-ceilometer-central):	Started myhost16
 Clone Set: openstack-ceilometer-api-clone [openstack-ceilometer-api]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: openstack-ceilometer-alarm-evaluator-clone [openstack-ceilometer-alarm-evaluator]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: openstack-ceilometer-collector-clone [openstack-ceilometer-collector]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: openstack-ceilometer-notification-clone [openstack-ceilometer-notification]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: openstack-ceilometer-alarm-notifier-clone [openstack-ceilometer-alarm-notifier]
     Started: [ myhost15 myhost16 myhost17 ]
 Clone Set: ceilometer-delay-clone [ceilometer-delay]
     Started: [ myhost15 myhost16 myhost17 ]
 openstack-nova-compute-vc13cl42	(systemd:openstack-nova-compute-vc13cl42):	Started myhost15
 openstack-manila-api	(systemd:openstack-manila-api):	Started myhost16
 openstack-manila-scheduler	(systemd:openstack-manila-scheduler):	Started myhost16
 openstack-manila-share	(systemd:openstack-manila-share):	Started myhost16
 ip-manila-10.50.218.150	(ocf::heartbeat:IPaddr2):	Started myhost16

PCSD Status:
  myhost15: Online
  myhost17: Online
  myhost16: Online

Daemon Status:
  corosync: active/enabled
  pacemaker: active/enabled
  pcsd: active/enabled
""".strip()

CLUSTER_NODES_AND_RESOURCES = """
Cluster name: tripleo_cluster
Last updated: Tue May  2 03:08:03 2017          Last change: Tue May  2 01:54:35 2017 by root via crm_resource on overcloud-controller-0
Stack: corosync
Current DC: overcloud-controller-0 (version 1.1.13-10.el7_2.2-44eb2dd) - partition with quorum
3 nodes and 112 resources configured

Online: [ overcloud-controller-0 overcloud-controller-1 overcloud-controller-2 ]

Full list of resources:

 ip-172.20.20.10        (ocf::heartbeat:IPaddr2):       Started overcloud-controller-0
 Clone Set: haproxy-clone [haproxy]
     Started: [ overcloud-controller-0 overcloud-controller-1 overcloud-controller-2 ]
 ip-10.67.16.160        (ocf::heartbeat:IPaddr2):       Started overcloud-controller-2
 ip-172.40.40.10        (ocf::heartbeat:IPaddr2):       Started overcloud-controller-0
 ip-172.50.50.10        (ocf::heartbeat:IPaddr2):       Started overcloud-controller-1
 ip-192.0.2.31  (ocf::heartbeat:IPaddr2):       Started overcloud-controller-0
 ip-172.20.20.11        (ocf::heartbeat:IPaddr2):       Started overcloud-controller-2
 Master/Slave Set: redis-master [redis]
     Masters: [ overcloud-controller-0 ]
     Slaves: [ overcloud-controller-1 overcloud-controller-2 ]
 Master/Slave Set: galera-master [galera]
     Masters: [ overcloud-controller-0 overcloud-controller-1 overcloud-controller-2 ]
 Clone Set: mongod-clone [mongod]
     Started: [ overcloud-controller-0 overcloud-controller-1 overcloud-controller-2 ]
 Clone Set: rabbitmq-clone [rabbitmq]
     Started: [ overcloud-controller-0 overcloud-controller-1 overcloud-controller-2 ]
 Clone Set: memcached-clone [memcached]
     Started: [ overcloud-controller-0 overcloud-controller-1 overcloud-controller-2 ]
 Clone Set: openstack-nova-scheduler-clone [openstack-nova-scheduler]
     Started: [ overcloud-controller-0 overcloud-controller-1 overcloud-controller-2 ]
 Clone Set: neutron-l3-agent-clone [neutron-l3-agent]
     Started: [ overcloud-controller-0 overcloud-controller-1 overcloud-controller-2 ]

PCSD Status:
  overcloud-controller-0: Online
  overcloud-controller-1: Online
  overcloud-controller-2: Online

Daemon Status:
  corosync: active/enabled
  pacemaker: active/enabled
  pcsd: active/enabled
"""

CLUSTER_NOT_RUNNING = """
Error: cluster is not currently running on this node
"""

CLUSTER_WARNINGS = """
WARNING: We don't know where, but something awful is going to happen
WARNING: This is another made-up warning, please supply real ones
"""

CLUSTER_UNCLEANNODE = """
Node control-2: UNCLEAN (online)
Online: [ control-0 control-1 ]
RemoteOnline: [ compute-0 compute-1 compute-2 compute-3 compute-4 compute-5 compute-6 compute-7 ]

PCSD Status:
  control-0: Online
  control-1: Online
  control-2: Online
"""


def test_pcs_status():
    pcs = PCSStatus(context_wrap(CLUSTER_NORMAL))
    assert pcs.nodes == ['myhost15', 'myhost17', 'myhost16']
    assert pcs.get('Stack') == 'corosync'
    assert pcs.get('Cluster name') == 'openstack'
    assert pcs.get('Current DC') == 'myhost15 (1) - partition with quorum'
    assert pcs.get("Nodes configured") == "3"
    assert pcs.get("Resources configured") == "143"
    assert pcs.get("Online") == "[ myhost15 myhost16 myhost17 ]"
    assert pcs.get('Nonexistent key') is None


def test_pcs_nodes_and_resources():
    pcs = PCSStatus(context_wrap(CLUSTER_NODES_AND_RESOURCES))
    assert pcs.nodes == ['overcloud-controller-0', 'overcloud-controller-1', 'overcloud-controller-2']
    # 3 nodes and 112 resources configured
    assert pcs.data['Nodes configured'] == '3'
    assert pcs.data['Resources configured'] == '112'
    assert pcs.data['Daemon Status'] == [
        'corosync: active/enabled',
        'pacemaker: active/enabled',
        'pcsd: active/enabled',
    ]


def test_cluster_not_running():
    pcs = PCSStatus(context_wrap(CLUSTER_NOT_RUNNING))
    assert pcs.nodes == []


def test_cluster_warning():
    pcs = PCSStatus(context_wrap(CLUSTER_WARNINGS))
    assert pcs.nodes == []
    assert pcs.data['WARNING'] == [
        "WARNING: We don't know where, but something awful is going to happen",
        "WARNING: This is another made-up warning, please supply real ones"
    ]


def test_pcs_uncleannode():
    pcs = PCSStatus(context_wrap(CLUSTER_UNCLEANNODE))
    assert pcs.nodes == ['control-0', 'control-1', 'control-2']
    assert pcs.bad_nodes == [{'status': 'UNCLEAN (online)', 'name': 'control-2'}]



from insights.parsers.odbc import ODBCIni, ODBCinstIni
from insights.tests import context_wrap

ODBC_INI = """
[myodbc5w]
Driver       = /usr/lib64/libmyodbc5w.so
Description  = DSN to MySQL server
SERVER       = localhost
NO_SSPS     = 1

[mysqlDSN]
Driver      = /usr/lib64/libmyodbc5.so
SERVER      = localhost

[myodbc]
Driver=MySQL
SERVER=localhost
#NO_SSPS=1
""".strip()


def test_odbc_ini():
    res = ODBCIni(context_wrap(ODBC_INI))
    assert res.data.get("mysqlDSN", "Driver") == "/usr/lib64/libmyodbc5.so"
    assert res.data.get("mysqlDSN", "SERVER") == "localhost"
    assert not res.has_option("mysqlDSN", 'NO_SSPS')

    assert len(res.items("myodbc5w")) == 4
    assert res.getint("myodbc5w", 'NO_SSPS') == 1
    assert res.getint("myodbc5w", 'No_Ssps') == 1
    assert res.getint("myodbc5w", 'NO_SSPS'.lower()) == 1

    assert 'myodbc' in res
    assert res.data.get("myodbc", "Driver") == "MySQL"
    assert not res.has_option("myodbc", 'NO_SSPS')


ODBCINST_INI = """
# Example driver definitions

# Driver from the postgresql-odbc package
# Setup from the unixODBC package
[PostgreSQL]
Description	= ODBC for PostgreSQL
Driver		= /usr/lib/psqlodbcw.so
Setup		= /usr/lib/libodbcpsqlS.so
Driver64	= /usr/lib64/psqlodbcw.so
Setup64		= /usr/lib64/libodbcpsqlS.so
FileUsage	= 1


# Driver from the mysql-connector-odbc package
# Setup from the unixODBC package
[MySQL]
Description	= ODBC for MySQL
Driver		= /usr/lib/libmyodbc5.so
Setup		= /usr/lib/libodbcmyS.so
Driver64	= /usr/lib64/libmyodbc5.so
Setup64		= /usr/lib64/libodbcmyS.so
FileUsage	= 1
""".strip()


def test_odbcinst_ini():
    res = ODBCinstIni(context_wrap(ODBCINST_INI))
    assert 'PostgreSQL' in res
    assert 'MySQL' in res
    assert 'XXSQL' not in res
    assert res.data.get("MySQL", "Driver64") == "/usr/lib64/libmyodbc5.so"



"""
Ps - command ``ps auxww`` and others
====================================

This module provides processing for the various outputs of the ``ps`` command.
"""
from .. import parser, CommandParser
from . import ParseException, parse_delimited_table, keyword_search
from insights.specs import Specs
from insights.core.filters import add_filter


def are_present(tags, line):
    """bool: Returns True if all tags are present in line."""
    return all(tag in line for tag in tags)


class Ps(CommandParser):
    """
    Template Class to parse ``ps`` command output.

    Raises:
        ParseException: Raised if the heading line (containing both ``user_name``
            and ``command_name``) is not found in the input.

    Attributes:
        data (list): List of dicts, where the keys in each dict are the
            column headers and each item in the list represents a process.
        running (set): Set of full command strings for each command
            including optional path and arguments, in order of listing in the
            `ps` output.
        cmd_names (set): Set of just the command names, minus any path or
            arguments.
        services (list): List of tuples in format (cmd names, user/uid/pid, raw_line) for
            each command.

    """
    command_name = "COMMAND_TEMPLATE"
    '''
    ``command_name`` is the name of the subclass specific command column from the header of
    ps output, the subclass must override it correspondingly
    '''
    user_name = "USER_TEMPLATE"
    '''
    ``user_name`` is the name of the subclass specificuser_name column from the header of
    ps output, the subclass must override it correspondingly
    '''
    max_splits = 0
    '''
    ``max_splits`` is the split number for the columns from the ps output,
    the subclass must override it correspondingly
    '''

    def __init__(self, *args, **kwargs):
        self.data = []
        self.running = set()
        self.cmd_names = set()
        self.services = []
        super(Ps, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        raw_line_key = "_line"
        header_line = next((l for l in content if are_present(tags=[self.user_name, self.command_name], line=l)), None)
        if header_line is not None:
            # parse_delimited_table allows short lines, but we specifically
            # want to ignore them.
            self.data = [
                row
                for row in parse_delimited_table(
                    content, heading_ignore=[header_line], max_splits=self.max_splits,
                    raw_line_key=raw_line_key
                )
                # skip the insights-client self grep process "grep -F .."
                if self.command_name in row and not row[self.command_name].startswith('grep -F ')
            ]
            # The above list comprehension assures all rows have a command.
            for proc in self.data:
                cmd = proc[self.command_name]
                self.running.add(cmd)
                cmd_name = cmd
                if cmd.startswith('/'):
                    cmd_name = cmd.split(None, 1)[0].split("/")[-1]
                elif ' ' in cmd:
                    cmd_name = cmd.split(None, 1)[0]
                proc["COMMAND_NAME"] = cmd_name
                self.cmd_names.add(cmd_name)
                proc["ARGS"] = cmd.split(" ", 1)[1] if " " in cmd else ""
                self.services.append((cmd_name, proc[self.user_name], proc[raw_line_key]))
                del proc[raw_line_key]
        else:
            raise ParseException(
                "{0}: Cannot find ps header line containing {1} and {2} in output".format(
                    self.__class__.__name__, self.user_name, self.command_name)
            )

    def __contains__(self, proc):
        return proc in self.running

    def __iter__(self):
        for row in self.data:
            yield row

    def running_pids(self):
        """
        Gives the list of process IDs in the order listed.

        Returns:
            list: the PIDs from the PID column.
        """
        return [row["PID"] for row in self.data if "PID" in row]

    def users(self, proc):
        """
        Searches for all users running a given command.  If the
        user column is not present then returns an empty dict.

        Returns:
            dict: each username as a key to a list of PIDs (as strings) that
            are running the given process.
            ``{}`` if neither ``USER`` nor ``UID`` is found or ``proc`` is not found.

        .. note::
           'proc' must match the entire command and arguments.
        """
        valid_user_columns = ['USER', 'UID']
        ret = {}
        if self.user_name in valid_user_columns:
            for row in self.data:
                if proc == row[self.command_name]:
                    if row[self.user_name] not in ret:
                        ret[row[self.user_name]] = []
                    ret[row[self.user_name]].append(row["PID"])
        return ret

    def fuzzy_match(self, proc):
        """
        Are there any commands that contain the given text?

        Returns:
            boolean: ``True`` if the word ``proc`` appears in the command column.

        .. note::
           'proc' can match anywhere in the command path, name or arguments.
        """
        return any(proc in row[self.command_name] for row in self.data)

    def number_occurences(self, proc):
        """
        Returns the number of occurencies of commands that contain given text

        Returns:
            int: The number of occurencies of commands with given text

        .. note::
           'proc' can match anywhere in the command path, name or arguments.
        """
        return len([True for row in self.data if proc in row[self.command_name]])

    def search(self, **kwargs):
        """
        Search the process list for matching rows based on key-value pairs.

        This uses the :py:func:`insights.parsers.keyword_search` function for
        searching; see its documentation for usage details.  If no search
        parameters are given, no rows are returned.

        Returns:
            list: A list of dictionaries of processes that match the given
            search criteria.

        Examples:
            >>> ps.search(COMMAND__contains='bash') == [
            ...    {'%MEM': '0.0', 'TTY': 'pts/3', 'VSZ': '108472', 'ARGS': '', 'PID': '20160', '%CPU': '0.0',
            ...     'START': '10:09', 'COMMAND': '/bin/bash', 'USER': 'user1', 'STAT': 'Ss', 'TIME': '0:00',
            ...     'COMMAND_NAME': 'bash', 'RSS': '1896'},
            ...    {'%MEM': '0.0', 'TTY': '?', 'VSZ': '9120', 'ARGS': '', 'PID': '20457', '%CPU': '0.0',
            ...     'START': '10:09', 'COMMAND': '/bin/bash', 'USER': 'root', 'STAT': 'Ss', 'TIME': '0:00',
            ...     'COMMAND_NAME': 'bash', 'RSS': '832'}
            ... ]
            True
            >>> ps.search(USER='root', COMMAND__contains='bash') == [
            ...    {'%MEM': '0.0', 'TTY': '?', 'VSZ': '9120', 'ARGS': '', 'PID': '20457', '%CPU': '0.0',
            ...     'START': '10:09', 'COMMAND': '/bin/bash', 'USER': 'root', 'STAT': 'Ss', 'TIME': '0:00',
            ...     'COMMAND_NAME': 'bash', 'RSS': '832'}
            ... ]
            True
            >>> ps.search(TTY='pts/3') == [
            ...    {'%MEM': '0.0', 'TTY': 'pts/3', 'VSZ': '108472', 'ARGS': '', 'PID': '20160', '%CPU': '0.0',
            ...     'START': '10:09', 'COMMAND': '/bin/bash', 'USER': 'user1', 'STAT': 'Ss', 'TIME': '0:00',
            ...     'COMMAND_NAME': 'bash', 'RSS': '1896'}
            ... ]
            True
            >>> ps.search(STAT__contains='Z') == [
            ...    {'%MEM': '0.0', 'TTY': '?', 'VSZ': '0', 'ARGS': '', 'PID': '1821', '%CPU': '0.0',
            ...     'START': 'May31', 'COMMAND': '[kondemand/0]', 'USER': 'root', 'STAT': 'Z', 'TIME': '0:29',
            ...     'COMMAND_NAME': '[kondemand/0]', 'RSS': '0'}
            ... ]
            True
        """
        return keyword_search(self.data, **kwargs)


add_filter(Specs.ps_auxww, "COMMAND")


@parser(Specs.ps_auxww)
class PsAuxww(Ps):
    """
    Class ``PsAuxww`` parses the output of the ``ps auxww`` command.  A small
    sample of the output of this command looks like::

        USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
        root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
        root      1661  0.0  0.0 126252  1392 ?        Ss   May31   0:04 /usr/sbin/crond -n
        root      1691  0.0  0.0  42688   172 ?        Ss   May31   0:00 /usr/sbin/rpc.mountd
        root      1821  0.0  0.0      0     0 ?        Z    May31   0:29 [kondemand/0]
        root      1864  0.0  0.0  18244   668 ?        Ss   May31   0:05 /usr/sbin/irqbalance --foreground
        user1    20160  0.0  0.0 108472  1896 pts/3    Ss   10:09   0:00 /bin/bash
        root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 /usr/sbin/dhclient enp0s25
        root     20457  0.0  0.0   9120   832 ?        Ss   10:09   0:00 /bin/bash

    PsAuxww attempts to read the output of ``ps auxwww``, ``ps aux``, and ``ps
    auxcww`` commands from archives.

    Examples:
        >>> type(ps_auxww)
        <class 'insights.parsers.ps.PsAuxww'>
        >>> ps_auxww.running == set([
        ...     '/bin/bash', '/usr/sbin/rpc.mountd', '/usr/lib/systemd/systemd --switched-root --system --deserialize 22',
        ...     '/usr/sbin/irqbalance --foreground', '/usr/sbin/dhclient enp0s25', '[kondemand/0]', '/usr/sbin/crond -n'
        ... ])
        True
        >>> ps_auxww.cpu_usage('[kondemand/0]')
        '0.0'
        >>> ps_auxww.users('/bin/bash') == {'root': ['20457'], 'user1': ['20160']}
        True
        >>> ps_auxww.fuzzy_match('dhclient')
        True
        >>> sum(int(p['VSZ']) for p in ps_auxww)
        333252
    """
    command_name = "COMMAND"
    user_name = "USER"
    max_splits = 10

    def cpu_usage(self, proc):
        """
        Searches for the first command matching ``proc`` and returns its
        CPU usage as a string.

        Returns:
            str: the %CPU column corresponding to ``proc`` in command or
            ``None`` if ``proc`` is not found.

        .. note::
           'proc' must match the entire command and arguments.
        """
        for row in self.data:
            if proc == row[self.command_name]:
                return row["%CPU"]

    pass


add_filter(Specs.ps_ef, "CMD")


@parser(Specs.ps_ef)
class PsEf(Ps):
    """
    Class ``PsEf`` parses the output of the ``ps -ef`` command.  A small
    sample of the output of this command looks like::

        UID         PID   PPID  C STIME TTY          TIME CMD
        root          1      0  0 03:53 ?        00:00:06 /usr/lib/systemd/systemd --system --deserialize 15
        root          2      0  0 03:53 ?        00:00:00 [kthreadd]
        root       1803      1  5 03:54 ?        00:55:22 /usr/bin/openshift start master --config=/etc/origin/master/master-config.yaml --loglevel
        root       1969      1  3 03:54 ?        00:33:51 /usr/bin/openshift start node --config=/etc/origin/node/node-config.yaml --loglevel=2
        root       1995      1  0 03:54 ?        00:02:06 /usr/libexec/docker/rhel-push-plugin
        root       2078   1969  0 03:54 ?        00:00:00 journalctl -k -f
        root       7201      1  0 03:59 ?        00:00:00 /usr/bin/python /usr/libexec/rhsmd
        root     111434      1  0 22:32 ?        00:00:00 nginx: master process /usr/sbin/nginx -c /etc/nginx/nginx.conf
        nginx    111435 111434  0 22:32 ?        00:00:00 nginx: worker process

    PsEf attempts to read the output of ``ps -ef`` commands from archives.

    Examples:
        >>> type(ps_ef)
        <class 'insights.parsers.ps.PsEf'>
        >>> ps_ef.parent_pid("111435")
        ['111434', 'nginx: master process /usr/sbin/nginx -c /etc/nginx/nginx.conf']
        >>> ps_ef.users('nginx: worker process')
        {'nginx': ['111435']}
        >>> ps_ef.fuzzy_match('kthreadd')
        True
    """
    command_name = "CMD"
    user_name = "UID"
    max_splits = 7

    def parent_pid(self, pid):
        """
        Search for the parent pid of command matching ``pid`` and returns
        the parent pid.

        Returns:
            list: First one is the parent pid corresponding to ``pid`` in command and second one is parent command name.
            ``None`` if ``proc`` is not found.
        """
        for row in self.data:
            if pid == row["PID"]:
                for sub_row in self.data:
                    if sub_row["PID"] == row["PPID"]:
                        return [row["PPID"], sub_row[self.command_name]]

    pass


@parser(Specs.ps_auxcww)
class PsAuxcww(PsAuxww):
    pass


add_filter(Specs.ps_aux, "COMMAND")


@parser(Specs.ps_aux)
class PsAux(PsAuxww):
    pass


@parser(Specs.ps_eo)
class PsEo(Ps):
    """
    Class to parse the command `ps -eo pid,ppid,comm`

    Sample input data::

          PID  PPID COMMAND
            1     0 systemd
            2     0 kthreadd
            3     2 ksoftirqd/0
         2416     1 auditd
         2419  2416 audispd
         2421  2419 sedispatch
         2892     1 NetworkManager
         3172  2892 dhclient
         3871     1 master
         3886  3871 qmgr
        13724  3871 pickup
        15663     2 kworker/0:1
        16998     2 kworker/0:3
        17259     2 kworker/0:0
        18294  3357 sshd

    Attributes:
        pid_info(dict): Dictionary with PID as key containing ps row as a dict

    Examples:
        >>> type(ps_eo)
        <class 'insights.parsers.ps.PsEo'>
        >>> ps_eo.pid_info['1'] == {'PID': '1', 'PPID': '0', 'COMMAND': 'systemd', 'COMMAND_NAME': 'systemd', 'ARGS': ''}
        True
        >>> ps_eo.children('2') == [
        ...     {'PID': '3', 'PPID': '2', 'COMMAND': 'ksoftirqd/0', 'COMMAND_NAME': 'ksoftirqd/0', 'ARGS': ''},
        ...     {'PID': '15663', 'PPID': '2', 'COMMAND': 'kworker/0:1', 'COMMAND_NAME': 'kworker/0:1', 'ARGS': ''},
        ...     {'PID': '16998', 'PPID': '2', 'COMMAND': 'kworker/0:3', 'COMMAND_NAME': 'kworker/0:3', 'ARGS': ''},
        ...     {'PID': '17259', 'PPID': '2', 'COMMAND': 'kworker/0:0', 'COMMAND_NAME': 'kworker/0:0', 'ARGS': ''}
        ... ]
        True
    """
    command_name = 'COMMAND'
    user_name = 'PID'
    max_splits = 3

    def parse_content(self, content):
        super(PsEo, self).parse_content(content)
        self.pid_info = {}
        for row in self.data:
            self.pid_info[row['PID']] = row

    def children(self, ppid):
        """list: Returns a list of dict for all rows with `ppid` as parent PID"""
        return [row for row in self.data if row['PPID'] == ppid]


add_filter(Specs.ps_alxwww, "COMMAND")


@parser(Specs.ps_alxwww)
class PsAlxwww(Ps):
    """
    Class to parse the command `ps alxwww`.  See method and attribute details
    in the ``Ps`` parser.

    Sample input data::

        F   UID   PID  PPID PRI  NI    VSZ   RSS WCHAN  STAT TTY        TIME COMMAND
        4     0     1     0  20   0 128292  6928 ep_pol Ss   ?          0:02 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
        1     0     2     0  20   0      0     0 kthrea S    ?          0:00 [kthreadd]
        1     0     3     2  20   0      0     0 smpboo S    ?          0:00 [ksoftirqd/0]
        5     0     4     2  20   0      0     0 worker S    ?          0:00 [kworker/0:0]
        1     0     5     2   0 -20      0     0 worker S<   ?          0:00 [kworker/0:0H]
        1     0     6     2  20   0      0     0 worker S    ?          0:00 [kworker/u4:0]
        1     0     7     2 -100  -      0     0 smpboo S    ?          0:00 [migration/0]
        1     0     8     2  20   0      0     0 rcu_gp S    ?          0:00 [rcu_bh]

    Examples:
        >>> type(ps_alxwww)
        <class 'insights.parsers.ps.PsAlxwww'>
        >>> 'systemd' in ps_alxwww.cmd_names
        True
        >>> '/usr/lib/systemd/systemd --switched-root --system --deserialize 22' in ps_alxwww.running
        True
        >>> ps_alxwww.search(COMMAND_NAME__contains='systemd') == [{
        ...     'F': '4', 'UID': '0', 'PID': '1', 'PPID': '0', 'PRI': '20', 'NI': '0', 'VSZ': '128292', 'RSS': '6928',
        ...     'WCHAN': 'ep_pol', 'STAT': 'Ss', 'TTY': '?', 'TIME': '0:02',
        ...     'COMMAND': '/usr/lib/systemd/systemd --switched-root --system --deserialize 22',
        ...     'COMMAND_NAME': 'systemd', 'ARGS': '--switched-root --system --deserialize 22'
        ... }]
        True

    """
    command_name = 'COMMAND'
    user_name = 'UID'
    max_splits = 12

    pass



"""
CephOsdTreeText - command ``ceph osd tree``
===========================================
"""

from insights.specs import Specs
from insights import parser, CommandParser, LegacyItemAccess
from insights.parsers import ParseException, SkipException

text_to_json_header_map = {
    "ID": "id",
    "CLASS": "device_class",
    "WEIGHT": "crush_weight",
    "TYPE NAME": "type_name",
    "TYPE_NAME": "type_name",
    "STATUS": "status",
    "UP/DOWN": "status",
    "REWEIGHT": "reweight",
    "PRI-AFF": "primary_affinity",
    "PRIMARY-AFFINITY": "primary_affinity"
}


@parser(Specs.ceph_osd_tree_text)
class CephOsdTreeText(CommandParser, LegacyItemAccess):
    """
    Class to parse the output of command ``ceph osd tree``.

    The typical content is::

        ID CLASS WEIGHT  TYPE NAME       STATUS REWEIGHT PRI-AFF
        -1       0.08752 root default
        -9       0.02917     host ceph1
         2   hdd 0.01459         osd.2       up  1.00000 1.00000
         5   hdd 0.01459         osd.5       up  1.00000 1.00000
        -5       0.02917     host ceph2
         1   hdd 0.01459         osd.1       up  1.00000 1.00000
         4   hdd 0.01459         osd.4       up  1.00000 1.00000
        -3       0.02917     host ceph3
         0   hdd 0.01459         osd.0       up  1.00000 1.00000
         3   hdd 0.01459         osd.3       up  1.00000 1.00000
        -7             0     host ceph_1

    Examples:

        >>> ceph_osd_tree_text['nodes'][0]['id']
        '-1'
        >>> ceph_osd_tree_text['nodes'][0]['name']
        'default'
        >>> ceph_osd_tree_text['nodes'][0]['type']
        'root'
        >>> ceph_osd_tree_text['nodes'][3]['type']
        'osd'
    """

    def parse_content(self, content):
        def calc_column_indices(line, headers):
            idx = []
            for h in headers:
                i = idx[-1] + 1 if idx else 0
                idx.append(line.index(h, i))
            return idx

        if not content:
            raise SkipException("Empty content.")
        if len(content) == 1 or "TYPE NAME" not in content[0]:
            raise ParseException("Wrong content in table: '{0}'.".format(content))

        header = content[0].replace('TYPE NAME', 'TYPE_NAME')

        col_headers = header.strip().split()
        json_headers = [text_to_json_header_map[h] for h in col_headers]
        col_index = calc_column_indices(header, col_headers)

        name_header_index = header.index("TYPE_NAME")
        parents = []
        nodes = []
        for row_num, line in enumerate(content[1:]):
            node = dict(
                (json_headers[c], line[col_index[c]:col_index[c + 1]].strip())
                for c in range(len(col_index) - 1)
            )
            node[json_headers[-1]] = line[col_index[-1]:].strip()

            # update parent's children list
            name_index = line.index(node['type_name'])
            shift = (name_index - name_header_index) / 4
            if shift == 0:
                pass
            elif shift > len(parents):
                prev_line = row_num - 1
                parents.append(prev_line)
                nodes[parents[-1]]['children'].insert(0, int(node['id']))
            elif shift == len(parents):
                nodes[parents[-1]]['children'].insert(0, int(node['id']))
            elif shift < len(parents):
                parents.pop()
                nodes[parents[-1]]['children'].insert(0, int(node['id']))
            # update type name
            type_name = node.pop('type_name').split()
            node['type'] = type_name[0] if len(type_name) > 1 else type_name[0].split('.')[0]
            node['name'] = type_name[-1]
            nodes.append(node)
            # update children list for the none leave node
            if len(type_name) > 1:
                node['children'] = []
        self.data = {'nodes': nodes}



"""
Tuned - command ``/usr/sbin/tuned-adm list``
============================================

This parser reads the output of the ``/usr/sbin/tuned-adm list`` command and
reads it into a simple dictionary in the ``data`` property with two of three
keys:

* ``available`` - the list of available profiles
* ``active`` - the active profile name
* ``preset`` - the profile name that's preset to be used when tuned is active

The ``active`` key is available when ``tuned`` is running, because the active
profile is only listed when the daemon is active.  If ``tuned`` is not
running, the tuned-adm command will list the profile that will be used when
the daemon is running, and this is given in the ``preset`` key.

Sample data::

    Available profiles:
    - balanced
    - desktop
    - latency-performance
    - network-latency
    - network-throughput
    - powersave
    - throughput-performance
    - virtual-guest
    - virtual-host
    Current active profile: virtual-guest

Examples:

    >>> result = shared[Tuned]
    >>> 'active' in result.data
    True
    >>> result.data['active']
    'virtual-guest'
    >>> len(result.data['available'])
    9
    >>> 'balanced' in result.data['available']
    True
"""

from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.tuned_adm)
class Tuned(CommandParser):
    """
    Parse data from the ``/usr/sbin/tuned-adm list`` command.
    """

    def parse_content(self, content):
        self.data = {}
        self.data['available'] = []
        for line in content:
            if line.startswith('-'):
                self.data['available'].append(line.split('- ')[1])
            elif line.startswith('Current'):
                self.data['active'] = line.split(': ')[1]
            elif line.startswith('Preset'):
                self.data['preset'] = line.split(': ')[1]
            # Ignore everything else for now



"""
php_ini - file ``/etc/php.ini``
===============================

This module provides the ``PHPConfig`` class parser, for reading the
options in the ``/etc/php.ini`` file.

Typical content of ``/etc/php.ini`` file is::

    [PHP]
    engine = On
    short_open_tag = Off
    precision = 14
    output_buffering = 4096
    zlib.output_compression = Off
    implicit_flush = Off
    unserialize_callback_func =
    serialize_precision = -1
    disable_functions =
    disable_classes =
    zend.enable_gc = On
    zend.exception_ignore_args = On
    zend.exception_string_param_max_len = 0
    expose_php = On
    max_execution_time = 30
    max_input_time = 60
    memory_limit = 128M
    error_reporting = E_ALL & ~E_DEPRECATED & ~E_STRICT
    default_mimetype = "text/html"

The class has one attribute ``data`` which is a nested ``dict`` representing sections
of the input INI file. Each section is represented as ``dict`` where keys are name
of options and values are values of those options.

Example:
    >>> php_conf["PHP"]["default_mimetype"].value
    'text/html'
    >>> php_config.data['PHP']['default_mimetype']
    'text/html'
    >>> php_conf.data['Session']['session.cache_limiter']
    'nocache'
    >>> php_conf["PHP"]["max_execution_time"].value
     30
    >>> php_conf["PHP"]["engine"].value  # 'On' turns to 'True'
    True
    >>> php_conf["PHP"]["short_opeh_tag"].value  # 'Off' turns to 'False'
    False
    >>> php_c['PHP']['precision'].value
    14
    >>> php_conf.get("PHP").get("memory_limit")  # '128M' is converted into bytes
    134217728
"""
import string
from insights import parser
from insights.core import ConfigParser
from insights.core.filters import add_filter
from insights.parsr.query import eq
from insights.parsr import (Char, EOF, HangingString, InSet,
        LeftBracket, Lift, LineEnd, Literal, RightBracket,
        Many, Number, OneLineComment, Opt, PosMarker,
        QuotedString, skip_none, String, WithIndent, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section
from insights.parsers import ParseException, SkipException
from insights.specs import Specs


# Filter to ensure that the section headings will always be included.
add_filter(Specs.php_ini, "[")


@parser(Specs.php_ini, continue_on_error=False)
class PHPConf(ConfigParser):
    """
    Class for php configuration file.
    """
    def parse_doc(self, content):
        try:
            def to_directive(x):
                name, rest = x
                rest = [rest] if rest is not None else []
                return Directive(name=name.value.strip(), attrs=rest, lineno=name.lineno, src=self)

            def to_section(name, rest):
                return Section(name=name.value.strip(), children=rest, lineno=name.lineno, src=self)

            def apply_defaults(cfg):
                if "DEFAULT" not in cfg:
                    return cfg

                defaults = cfg["DEFAULT"]
                not_defaults = cfg[~eq("DEFAULT")]
                for c in not_defaults:
                    for d in defaults.grandchildren:
                        if d.name not in c:
                            c.children.append(d)

                cfg.children = list(not_defaults)
                return cfg

            def make_bytes(number, char_multiple):
                if char_multiple.lower() == 'k':
                    return number * 2**10
                if char_multiple.lower() == 'm':
                    return number * 2**20
                if char_multiple.lower() == 'g':
                    return number * 2**30

            content = "\n".join(content)

            header_chars = (set(string.printable) - set(string.whitespace) - set("[]")) | set(" ")
            sep_chars = set("=")
            key_chars = header_chars - sep_chars
            value_chars = set(string.printable) - set("\n\r")

            On = Literal("on", True, ignore_case=True)
            Off = Literal("off", False, ignore_case=True)
            Tru = Literal("true", True, ignore_case=True)
            Fals = Literal("false", False, ignore_case=True)
            Boolean = ((On | Off | Tru | Fals) & (WSChar | LineEnd)) % "Boolean"
            Num = Number & (WSChar | LineEnd)
            QuoStr = QuotedString & (WSChar | LineEnd)
            # Handle php.ini shorthand notation for memory limits: 1G, 8M, 50K
            # https://www.php.net/manual/en/faq.using.php#faq.using.shorthandbytes
            MemNum = (Lift(make_bytes) * Number * (Char('K') | Char('M') | Char('G'))) & (WSChar | LineEnd)

            LeftEnd = (WS + LeftBracket + WS)
            RightEnd = (WS + RightBracket + WS)
            Header = (LeftEnd >> PosMarker(String(header_chars)) << RightEnd) % "Header"
            Key = WS >> PosMarker(String(key_chars)) << WS
            Sep = InSet(sep_chars, "Sep")
            Value = WS >> (Boolean | MemNum | Num | QuoStr | HangingString(value_chars))
            KVPair = WithIndent(Key + Opt(Sep >> Value)) % "KVPair"
            Comment = (WS >> (OneLineComment(";")).map(lambda x: None))

            Line = Comment | KVPair.map(to_directive)
            Sect = Lift(to_section) * Header * Many(Line).map(skip_none)
            Doc = Many(Comment | Sect).map(skip_none)
            Top = Doc << WS << EOF

            res = Entry(children=Top(content), src=self)
            return apply_defaults(res)

        except SkipException:
            raise
        except:
            raise ParseException(ParseException("Could not parse content: '{0}'".
                                                format(content)))

    def parse_content(self, content):
        super(PHPConf, self).parse_content(content)
        dict_all = {}
        for section in self.doc:
            section_dict = {}
            option_names = set(o.name for o in section)
            for name in option_names:
                options = [str(o.value) for o in section[name]]
                section_dict[name] = options[0] if len(options) == 1 else options
            dict_all[section.name] = section_dict
        self.data = dict_all



"""
HeatConf - file ``/etc/heat/heat.conf``
=======================================

This module provides plugins access to the heat.conf information.

Typical content of the ``heat.conf`` is::

    [DEFAULT]
    heat_metadata_server_url = http://172.16.0.11:8000
    heat_waitcondition_server_url = http://172.16.0.11:8000/v1/waitcondition
    heat_watch_server_url =http://172.16.0.11:8003
    stack_user_domain_name = heat_stack
    stack_domain_admin = heat_stack_domain_admin
    stack_domain_admin_password = *********
    auth_encryption_key = V48p9fRZzWSRgjE96e2I1oGwn216xgqf
    log_dir = /var/log/heat
    instance_user=
    notification_driver=messaging
    [auth_password]
    [clients]
    [clients_ceilometer]
    [clients_cinder]
    [clients_glance]
    [clients_heat]
    [clients_keystone]
    auth_uri =http://192.0.2.18:35357
    [clients_neutron]

Usage of this parser is similar to others that use the ``IniConfigFile`` base
class.

Examples:

    >>> conf = shared(HeatConf)
    >>> 'DEFAULT' in conf
    True
    >>> conf.get_item('clients_keystone', 'auth_uri')
    'http://192.0.2.18:35357'
"""
from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.heat_conf)
class HeatConf(IniConfigFile):
    """Parses content of "/etc/heat/heat.conf". """
    pass
